SystemOrganization addCategory: #'Zippers-Support'!
SystemOrganization addCategory: #Zippers!
SystemOrganization addCategory: #'Zippers-BST'!
SystemOrganization addCategory: #'Zippers-Tests'!

Object subclass: #BaseZipper
	instanceVariableNames: 'focus trail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!BaseZipper commentStamp: 'fbs 1/21/2012 09:49' prior: 0!
I am a zipper for navigating over an hierarchical structure.

That is, I allow one to navigate arbitrarily over a tree structure, and change parts of it, where "change" means "return a structure that shares part of the original structure, but otherwise leaves the host structure unchanged".

 My subclasses define the three structure-specific pieces I need, namely,
* can this node have children? (#branch)
* what are this node's children? (#children)
* how can I construct a new node, given a parent and some children nodes? (#newFocusOn:children)!

!BaseZipper class methodsFor: 'instance creation' stamp: 'fbs 2/4/2012 22:43'!
on: aTree
	^ self new focus: aTree trail: ZipperContext root.! !

!BaseZipper methodsFor: 'mutating' stamp: 'fbs 12/20/2011 21:23'!
change: aBlock
	"aBlock is a unary block whose evaluated value we use to replace the current focus."
	^ self mutate: (aBlock value: focus value) with: #safeChangeTo:.! !

!BaseZipper methodsFor: 'mutating' stamp: 'fbs 12/20/2011 21:23'!
changeTo: newValue
	^ self mutate: newValue with: #safeChangeTo:.! !

!BaseZipper methodsFor: 'private navigating' stamp: 'fbs 2/4/2012 22:11'!
changedRoot
	"Return the possibly new/mutated tree under navigation"
	^ trail isRoot
		ifTrue: [focus]
		ifFalse: [self changedUp changedRoot].! !

!BaseZipper methodsFor: 'private navigating' stamp: 'fbs 10/19/2011 17:25'!
changedUp
	^ self move: #safeChangedUp.! !

!BaseZipper methodsFor: 'pluggable behaviours' stamp: 'fbs 11/12/2011 20:56'!
children
	self subclassResponsibility.! !

!BaseZipper methodsFor: 'copying' stamp: 'fbs 6/6/2011 10:28'!
copy
	^ self class new
		focus: self focus
		trail: self trail.! !

!BaseZipper methodsFor: 'mutating' stamp: 'fbs 12/20/2011 21:36'!
delete
	^ self move: #safeDelete.! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:33'!
down
	"Return a Zipper focusing on the first child element of this node."
	^ self move: #safeDown.! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:37'!
downOrElse: aBlock
	^ self move: #safeDown orElse: aBlock.! !

!BaseZipper methodsFor: 'accessing' stamp: 'fbs 12/30/2010 22:06'!
focus
	^ focus.! !

!BaseZipper methodsFor: 'private' stamp: 'fbs 12/31/2010 16:37'!
focus: aTree trail: aTreeZipperNode
	focus := aTree.
	trail := aTreeZipperNode.! !

!BaseZipper methodsFor: 'mutating' stamp: 'fbs 12/20/2011 21:15'!
insertDown: anObject
	"Insert anObject as a child node of the current focus. Sometimes you need a 'spacer' in your tree: a binary tree needs one so that the node can have just a right subtree. In that event, we insert a ZTree empty."

	^ self mutate: anObject with: #safeInsertDown:.! !

!BaseZipper methodsFor: 'mutating' stamp: 'fbs 12/20/2011 21:15'!
insertLeft: anObject
	^ self mutate: anObject with: #safeInsertLeft:.! !

!BaseZipper methodsFor: 'mutating' stamp: 'fbs 12/20/2011 21:16'!
insertRight: anObject
	^ self mutate: anObject with: #safeInsertRight:.! !

!BaseZipper methodsFor: 'pluggable behaviours' stamp: 'fbs 11/12/2011 21:00'!
isBranch
	self subclassResponsibility.! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:34'!
left
	^ self move: #safeLeft.! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:36'!
leftOrElse: aBlock
	^ self move: #safeLeft orElse: aBlock.! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 12/22/2011 14:00'!
leftmost
	^ self move: #safeLeftmost.! !

!BaseZipper methodsFor: 'private' stamp: 'fbs 2/5/2012 08:42'!
move: directionSelector
	"Move in some direction. If we fall off the data structure, raise an exception."
	| move |
	move := self perform: directionSelector.
	^ move
		ifLeft: [ZipperNavigation signal: move value error]
		ifRight: [move value]! !

!BaseZipper methodsFor: 'private' stamp: 'fbs 2/5/2012 08:43'!
move: directionSelector orElse: aBlock
	"Move in some direction. If we can't, run aBlock. If aBlock takes parameters, pass in the current focus as an argument."
	| move |
	move := self perform: directionSelector.
	^ move
		ifRight: [ move value ]
		ifLeft: [ aBlock cull: self focus ].! !

!BaseZipper methodsFor: 'private' stamp: 'fbs 2/5/2012 21:35'!
mutate: anObject with: actionSelector
	| attempt |
	attempt := self perform: actionSelector with: anObject.
	^ attempt
		ifRight: [attempt value]
		ifLeft: [ZipperNavigation signal: attempt value error]! !

!BaseZipper methodsFor: 'private' stamp: 'fbs 1/21/2012 09:49'!
newFocusOn: anObject
	^ self newFocusOn: anObject children: #().! !

!BaseZipper methodsFor: 'pluggable behaviours' stamp: 'fbs 11/12/2011 20:57'!
newFocusOn: anObject children: aCollection
	self subclassResponsibility.! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:34'!
right
	^ self move: #safeRight.! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:36'!
rightOrElse: aBlock
	^ self move: #safeRight orElse: aBlock.! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 12/22/2011 14:02'!
rightmost
	^ self move: #safeRightmost! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 2/4/2012 22:11'!
root
	"Return the possibly new/mutated tree under navigation"
	^ trail isRoot
		ifTrue: [focus]
		ifFalse: [trail changed
			ifTrue: [self changedUp changedRoot]
			ifFalse: [self up root]]! !

!BaseZipper methodsFor: 'mutating' stamp: 'fbs 12/20/2011 21:25'!
safeChange: aBlock
	^ self safeChangeTo: (aBlock value: focus value).! !

!BaseZipper methodsFor: 'mutating' stamp: 'fbs 2/5/2012 08:28'!
safeChangeTo: newValue
	^ Right value: (self class new
		focus: newValue
		trail: (trail class "This could be a normal context or the root context."
			path: trail path
			leftNodes: trail leftNodes
			rightNodes: trail rightNodes
			parentNodes: trail parentNodes
			changed: true))! !

!BaseZipper methodsFor: 'private navigating' stamp: 'fbs 2/5/2012 08:34'!
safeChangedUp
	"When part of the structure has changed, return a pair (successfully moved, zipper on new location)."
	trail isRoot ifTrue: [^ Left value: (ZipperError error: #upAtRootNode location: self)].
	
	^ Right value: (self class new
					focus: (self
						newFocusOn: trail parentNode
						children: trail leftNodes reverse, {focus} , trail rightNodes)
					trail: trail path).! !

!BaseZipper methodsFor: 'mutating' stamp: 'fbs 2/5/2012 08:30'!
safeDelete
	trail isRoot ifTrue: [^ Left value: (ZipperError error: #deleteAtRoot location: self)].
	
	(trail leftNodes isEmpty and: [trail rightNodes isEmpty]) ifTrue:
		[^ Right value: (self class new
			focus: (self newFocusOn: trail parentNode)
			trail: trail path)].
			
	trail leftNodes isEmpty ifTrue:
		[^ Right value: (self class new
			focus: trail rightNodes first
			trail: (ZipperContext
				path: trail path
				leftNodes: trail leftNodes
				rightNodes: trail rightNodes allButFirst
				parentNodes: trail parentNodes
				changed: true))].
	
	^ Right value: (self class new
		focus: trail leftNodes last
		trail: (ZipperContext
			path: trail path
			leftNodes: trail leftNodes allButLast
			rightNodes: trail rightNodes
			parentNodes: trail parentNodes
			changed: true)).! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 2/5/2012 21:58'!
safeDown
	"Return a pair whose first element tells us whether we successfully went down (represented by the value #success), and a Zipper. If yes, a Zipper focusing on the first child element of this node. If not, a Zipper focused on where we are."
	^ self isBranch
		ifTrue: [Right value: 
			(self class new
			focus: focus children first
			trail: (ZipperContext
					path: trail
					leftNodes: {}
					rightNodes: focus children allButFirst
					parentNodes: trail parentNodes, {focus}
					changed: false))]
		ifFalse: [Left value: (ZipperError error: #downAtLeafNode location: self)].! !

!BaseZipper methodsFor: 'mutating' stamp: 'fbs 2/5/2012 08:32'!
safeInsertDown: anObject
	"Insert anObject as a child node of the current focus. Sometimes you need a 'spacer' in your tree: a binary tree needs one so that the node can have just a right subtree. In that event, we insert a ZTree empty."

	focus isEmpty ifTrue: [^ self safeChangeTo: anObject].
	
	^ Right value: (self class new
			focus: (anObject = ZTree empty
				ifTrue: [ ZTree empty ]
				ifFalse: [ self newFocusOn: anObject ])
			trail: (ZipperContext
				path: trail
				leftNodes: {}
				rightNodes: focus children
				parentNodes: trail parentNodes, {focus}
				changed: true))! !

!BaseZipper methodsFor: 'mutating' stamp: 'fbs 2/5/2012 08:32'!
safeInsertLeft: anObject
	trail isRoot ifTrue: [^ Left value: (ZipperError error: #insertLeftAtRoot location: self)].

	^ Right value: (self class new
				focus: (self newFocusOn: anObject)
				trail: (ZipperContext
					path: trail path
					leftNodes: trail leftNodes
					rightNodes: ({focus} , trail rightNodes)
					parentNodes: trail parentNodes
					changed: true)).! !

!BaseZipper methodsFor: 'mutating' stamp: 'fbs 2/5/2012 08:33'!
safeInsertRight: anObject
	trail isRoot ifTrue: [^ Left value: (ZipperError error: #insertRightAtRoot location: self)].
		
	^ Right value: (self class new
			focus: (self newFocusOn: anObject)
			trail: (ZipperContext
				path: trail path
				leftNodes: (trail leftNodes , {focus})
				rightNodes: trail rightNodes
				parentNodes: trail parentNodes
				changed: true)).! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 2/5/2012 08:23'!
safeLeft
	"Return a pair (successfully moved, zipper on new location)."
	trail isRoot ifTrue: [^ Left value: (ZipperError error: #leftAtRootNode location: self)].
	trail leftNodes ifEmpty: [^ Left value: (ZipperError error: #leftAtRootNode location: self)].

	^ Right value: (self class new
			focus: trail leftNodes last
			trail: (ZipperContext
				path: trail path
				leftNodes: (trail leftNodes allButLast)
				rightNodes: ({focus} , trail rightNodes)
				parentNodes: trail parentNodes
				changed: false)).! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 2/5/2012 08:24'!
safeLeftmost
	"Return a pair (successfully moved, zipper on new location)."
	trail isRoot ifTrue: [^ Left value: (ZipperError error: #leftAtRootNode location: self)].
	trail leftNodes ifEmpty: [^ Left value: (ZipperError error: #leftAtLeftmostSibling location: self)].

	^ Right value: (self class new
			focus: trail leftNodes first
			trail: (ZipperContext
				path: trail path
				leftNodes: #()
				rightNodes: (trail leftNodes allButFirst, {focus} , trail rightNodes)
				parentNodes: trail parentNodes
				changed: false)).! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 2/5/2012 08:26'!
safeRight
	"Return a pair (successfully moved, zipper on new location)."
	trail isRoot ifTrue: [^ Left value: (ZipperError error: #rightAtRootNode location: self)].
	trail rightNodes ifEmpty: [^ Left value: (ZipperError error: #rightAtRightmostSibling location: self)].

	^ Right value: (self class new
			focus: trail rightNodes first
			trail: (ZipperContext
				path: trail path
				leftNodes: (trail leftNodes, {focus})
				rightNodes: trail rightNodes allButFirst
				parentNodes: trail parentNodes
				changed: false)).! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 2/5/2012 21:58'!
safeRightmost
	"Return a pair (successfully moved, zipper on new location)."
	trail isRoot ifTrue: [^ Left value: (ZipperError error: #rightAtRootNode location: self)].
	trail rightNodes ifEmpty: [^ Left value: (ZipperError error: #rightAtRightmostSibling location: self)].
	
	^ Right value: (self class new
			focus: trail rightNodes last
			trail: (ZipperContext
				path: trail path
				leftNodes: (trail leftNodes, {focus}, trail rightNodes allButLast)
				rightNodes: #()
				parentNodes: trail parentNodes
				changed: false)).! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 2/5/2012 08:27'!
safeUp
	"When no part of (this local part of) the structure has changed, return a pair (successfully moved, zipper on identical location)."
	trail isRoot ifTrue: [ ^ Left value: (ZipperError error: #upAtRootNode location: self)].

	^ Right value: (self class new
		focus: trail parentNodes last
		trail: trail path).! !

!BaseZipper methodsFor: 'accessing' stamp: 'fbs 2/16/2011 07:45'!
trail
	^ trail.! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:34'!
up
	^ self move: #safeUp.! !

!BaseZipper methodsFor: 'navigating' stamp: 'fbs 2/16/2011 17:35'!
upOrElse: aBlock
	^ self move: #safeUp orElse: aBlock.! !

BaseZipper subclass: #PluggableZipper
	instanceVariableNames: 'childrenBlock branchBlock mkFocusBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!PluggableZipper class methodsFor: 'instance creation' stamp: 'fbs 2/6/2012 19:02'!
on: aTree childrenBlock: childrenBlock branchBlock: branchBlock newNodeBlock: newNodeBlock
	^ self new
		focus: aTree
		trail: ZipperContext root
		childrenBlock: childrenBlock
		branchBlock: branchBlock
		newNodeBlock: newNodeBlock.! !

!PluggableZipper methodsFor: 'pluggable behaviours' stamp: 'fbs 2/6/2012 19:00'!
children
	^ childrenBlock value: self focus.! !

!PluggableZipper methodsFor: 'private' stamp: 'fbs 2/6/2012 19:04'!
focus: aTree trail: aZipperContext childrenBlock: unaryBlock1 branchBlock: unaryBlock2 newNodeBlock: binaryBlock.
	self focus: aTree trail: aZipperContext.
	childrenBlock := unaryBlock1.
	branchBlock := unaryBlock2.
	mkFocusBlock := binaryBlock.! !

!PluggableZipper methodsFor: 'pluggable behaviours' stamp: 'fbs 2/6/2012 19:00'!
isBranch
	^ branchBlock value: self focus! !

!PluggableZipper methodsFor: 'pluggable behaviours' stamp: 'fbs 2/6/2012 19:00'!
newFocusOn: anObject children: someObjects
	^ mkFocusBlock value: anObject value: someObjects.! !

BaseZipper subclass: #TreeZipper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!TreeZipper commentStamp: 'fbs 1/1/2011 18:01' prior: 0!
I am a zipper for navigating over a variable-arity tree, that is a tree that may have any number of children per node.

That is, I allow one to navigate arbitrarily over a tree structure (a ZTree), and change parts of it, where "change" means "return a structure that shares part of the original structure, but otherwise leaves the host structure unchanged".!

!TreeZipper methodsFor: 'pluggable behaviours' stamp: 'fbs 10/17/2011 08:20'!
children
	^ self focus children! !

!TreeZipper methodsFor: 'pluggable behaviours' stamp: 'fbs 10/15/2011 23:11'!
isBranch
	^ self focus children notEmpty! !

!TreeZipper methodsFor: 'pluggable behaviours' stamp: 'fbs 11/12/2011 20:56'!
isEmpty
	^ focus isEmpty.! !

!TreeZipper methodsFor: 'pluggable behaviours' stamp: 'fbs 12/20/2011 16:13'!
newFocusOn: anObject children: aCollection
	^ (anObject isKindOf: ZTree)
		ifTrue: [ZTree value: anObject value children: aCollection]
		ifFalse: [ZTree value: anObject children: aCollection].! !

Object subclass: #BinarySearchTree
	instanceVariableNames: 'tree'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-BST'!

!BinarySearchTree commentStamp: 'fbs 2/28/2011 22:27' prior: 0!
I implement a binary search tree in a strictly functional manner. I'm pretty naive, in that I do no balancing.!

!BinarySearchTree class methodsFor: 'instance creation' stamp: 'fbs 4/28/2011 12:46'!
tree: aBinaryTree
	^ self new tree: aBinaryTree.! !

!BinarySearchTree class methodsFor: 'instance creation' stamp: 'fbs 4/30/2011 17:08'!
with: anObject
	^ self new tree: (ZTree value: anObject).! !

!BinarySearchTree methodsFor: 'adding' stamp: 'fbs 12/20/2011 16:40'!
add: aComparableObject
	"Add some object that understands < to our tree. aComparableObject < node goes into the left subtree; otherwise, the object goes into the right subtree."
	^ BinarySearchTree tree: (self add: aComparableObject to: tree zipper) root! !

!BinarySearchTree methodsFor: 'private' stamp: 'fbs 2/5/2012 22:40'!
add: anObject to: aZipper
	"Return a zipper with anObject inserted in the correct place: if we're at a leaf node, and <, insertDown; otherwise, move further into the tree."
	| currentValue |
	aZipper focus isEmpty ifTrue: [ ^ aZipper insertDown: (ZTree value: anObject) ].
	
	currentValue := aZipper focus value.

	aZipper safeDown
		either: [:leftChild | "Move down to the left subnode."
			"If less than, add to the left subtree."
			anObject < currentValue
				ifTrue: [ ^ self add: anObject to: leftChild ].
		
			"Otherwise, add to the right subtree."
			leftChild safeRight
				either: [:rightChild | ^ self add: anObject to: rightChild]
				or: [:error | "no right subtree" ^ aZipper insertDown: (ZTree value: anObject)].
			]
		or: [:error | "Leaf node. If adding a right subtree, add the empty tree as a left subtree."
			^ anObject < currentValue
				ifTrue: [aZipper insertDown: (ZTree value: anObject) ]
				ifFalse: [(aZipper insertDown: ZTree empty) insertRight: (ZTree value: anObject)]].! !

!BinarySearchTree methodsFor: 'printing' stamp: 'fbs 4/24/2011 09:30'!
asDotFile
	^ tree asDotFile! !

!BinarySearchTree methodsFor: 'converting' stamp: 'fbs 4/30/2011 22:30'!
asOrderedCollection
	^ tree asOrderedCollection.! !

!BinarySearchTree methodsFor: 'enumerating' stamp: 'fbs 4/30/2011 22:30'!
collect: aBlock
	^ tree collect: aBlock.! !

!BinarySearchTree methodsFor: 'initialize-release' stamp: 'fbs 2/24/2011 17:33'!
initialize
	tree := ZTree empty.! !

!BinarySearchTree methodsFor: 'printing' stamp: 'fbs 4/22/2011 22:13'!
printString
	^ tree printString.! !

!BinarySearchTree methodsFor: 'converting' stamp: 'fbs 4/30/2011 17:17'!
rebalanced
	^ self class tree: self asOrderedCollection asBalancedBinaryTree.! !

!BinarySearchTree methodsFor: 'converting' stamp: 'fbs 4/29/2011 19:10'!
rebalancedDSW
	^ self class tree: tree balanceDSW.! !

!BinarySearchTree methodsFor: 'converting' stamp: 'fbs 5/1/2011 10:16'!
rebalancedDSW2
	^ self class tree: tree balanceDSW2.! !

!BinarySearchTree methodsFor: 'private' stamp: 'fbs 2/21/2011 22:12'!
tree: aTree
	tree := aTree.! !

!BinarySearchTree methodsFor: 'accessing' stamp: 'fbs 2/28/2011 22:26'!
zipper
	"This lets me masquerade as a ZTree: PreOrderTraversal new traverse: aBST collecting: [:each | each * 2 ]."
	^ tree zipper.! !

Object subclass: #BinaryTreeZipper
	instanceVariableNames: 'internalZipper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!BinaryTreeZipper commentStamp: 'fbs 4/30/2011 16:12' prior: 0!
I am a zipper that walks over a binary tree. I differ from TreeZipper in my semantics:

  - my "left" means "down" in a TreeZipper
  - my "right" means "down right" in a TreeZipper
  - my "down" means "down" in a TreeZipper (to seamlessly support the Traversals)

I wrap a normal TreeZipper, implementing the above semantic difference.!

!BinaryTreeZipper class methodsFor: 'instance creation' stamp: 'fbs 4/30/2011 15:18'!
on: aTree
	^ self new using: aTree zipper.! !

!BinaryTreeZipper methodsFor: 'accessing' stamp: 'fbs 4/30/2011 15:18'!
focus
	^ internalZipper focus.! !

!BinaryTreeZipper methodsFor: 'mutating' stamp: 'fbs 11/1/2011 08:00'!
insertDown: anObject
	^ internalZipper focus isEmpty
		ifTrue: [internalZipper changeTo: anObject]
		ifFalse: [internalZipper insertDown: anObject].! !

!BinaryTreeZipper methodsFor: 'mutating' stamp: 'fbs 4/30/2011 16:37'!
insertLeft: anObject
	"Insert anObject as the left subtree, and focus on this new node."
	^ internalZipper insertDown: anObject.! !

!BinaryTreeZipper methodsFor: 'mutating' stamp: 'fbs 4/30/2011 17:07'!
insertRight: anObject
	"Insert anObject as the right subtree, and focus on this new node."
	"If this is the empty tree, 'insert' means 'make a node'."
	internalZipper isEmpty ifTrue: [ ^ internalZipper insertDown: anObject ].
	
	^ (internalZipper focus hasChildren
		ifTrue: [internalZipper down]
		ifFalse: [internalZipper insertDown: ZTree empty])
			insertRight: anObject.! !

!BinaryTreeZipper methodsFor: 'navigating' stamp: 'fbs 4/30/2011 15:25'!
left
	^ internalZipper down.! !

!BinaryTreeZipper methodsFor: 'navigating' stamp: 'fbs 4/30/2011 15:37'!
right
	"'Right subtree' for a ZTree means 'second child'."
	internalZipper focus hasChildren ifFalse: [ ZipperNavigation signal: #noRightSubtree ].
	^ internalZipper down right.! !

!BinaryTreeZipper methodsFor: 'navigating' stamp: 'fbs 4/30/2011 09:29'!
root
	"Return the root of the (possibly) mutated tree."! !

!BinaryTreeZipper methodsFor: 'navigating' stamp: 'fbs 4/30/2011 16:11'!
safeDown
	^ self safeLeft.! !

!BinaryTreeZipper methodsFor: 'navigating' stamp: 'fbs 2/5/2012 22:33'!
safeLeft
	"Return a pair (successfully moved, zipper on new location)."
	"'Left' means 'down to the first child'."
	^ internalZipper safeDown
		either: [:z | z focus isEmpty
			ifTrue: [Left value: (ZipperError error: #noLeftSubtree location: self)]
			ifFalse: [Right value: z]]
		or: [:error | Left value: (ZipperError error: #noLeftSubtree location: self)].! !

!BinaryTreeZipper methodsFor: 'navigating' stamp: 'fbs 2/5/2012 22:29'!
safeRight
	"Return a pair (successfully moved, zipper on new location)."
	^ internalZipper safeDown
		either: [:leftChild |
			leftChild safeRight
				either: [:rightChild |
					rightChild value focus isEmpty
						ifTrue: [Left value: (ZipperError error: #noRightSubtree location: self)]
						ifFalse: [Right value: rightChild]]
				or: [:error | Left value: (ZipperError error: #noRightSubtree location: self)]]
		or: [:error | Left value: (ZipperError error: #rightAtLeafNode location: self)]! !

!BinaryTreeZipper methodsFor: 'navigating' stamp: 'fbs 4/30/2011 16:03'!
safeUp
	^ internalZipper safeUp.! !

!BinaryTreeZipper methodsFor: 'navigating' stamp: 'fbs 4/30/2011 16:01'!
up
	^ internalZipper up.! !

!BinaryTreeZipper methodsFor: 'private' stamp: 'fbs 4/30/2011 15:18'!
using: aZipper
	internalZipper := aZipper.! !

Object subclass: #BreadthFirstSearch
	instanceVariableNames: 'queue visited zipper'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!BreadthFirstSearch commentStamp: 'fbs 2/6/2012 09:38' prior: 0!
A breadth first traversal processes nodes in order of depth, and left to right.

Since a zipper excels at _local_ movement in a structure, and since I of necessity must use nonlocal movement (once it has visited all nodes in any depth n, it must leap to the leftmost node at depth n+1), I spawn zippers for each child node.

This means that unlike my depth-first cousins (which all use a single zipper), I will _not_ preserve mutations on my structure except for the very last zipper I spawn - that of the rightmost deepest node.!

!BreadthFirstSearch methodsFor: 'private' stamp: 'fbs 2/6/2012 09:26'!
addUnmarkedChildren: aZipper
	self trampoline: aZipper safeDown into: [ :zl |
		zl
			either: [ :z |
				(visited includes: (z focus))
					ifFalse: [queue addLast: z].
				[z safeRight]]
			or: [ :noChildrenError | noChildrenError location]]! !

!BreadthFirstSearch methodsFor: 'enumerating' stamp: 'fbs 2/6/2012 09:28'!
collect: aBlock
	| prev |
	prev := zipper.
	[self hasNext] whileTrue:
		[zipper := zipper changeTo: (aBlock value: zipper focus).
		prev := zipper.
		zipper := self next].
	^ prev.! !

!BreadthFirstSearch methodsFor: 'private' stamp: 'fbs 2/4/2012 23:26'!
hasNext
	^ zipper trail isEnd not! !

!BreadthFirstSearch methodsFor: 'private' stamp: 'fbs 2/4/2012 23:37'!
next
	self hasNext ifFalse: [^ zipper].
	queue ifEmpty: [^ zipper := zipper class new
		focus: zipper value
		trail: (EndOfTraversalContext trail: zipper trail)].
	
	visited addLast: zipper value.
	
	zipper := queue first.
	queue := queue allButFirst.
	self addUnmarkedChildren: zipper.
	^ zipper.! !

!BreadthFirstSearch methodsFor: 'private' stamp: 'fbs 2/6/2012 09:25'!
trampoline: initialValue into: aUnaryBlock
	| v |
	v := [initialValue].
	[v isBlock] whileTrue: [v := aUnaryBlock value: v value].
	^ v.! !

!BreadthFirstSearch methodsFor: 'enumerating' stamp: 'fbs 2/6/2012 09:29'!
traverse: anObject collecting: aBlock
	queue := OrderedCollection new.
	visited := OrderedCollection new.
	zipper := anObject zipper.
	self addUnmarkedChildren: zipper.
	
	^ self collect: aBlock.! !

!BreadthFirstSearch methodsFor: 'enumerating' stamp: 'fbs 2/4/2012 23:38'!
traverse: anObject doing: aBlock
	"Throw away the return value."
	self traverse: anObject collecting: aBlock.! !

Object subclass: #DepthTraversal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!DepthTraversal commentStamp: 'fbs 2/26/2011 23:06' prior: 0!
My subclasses define various ways of traversing trees and graphs.!

!DepthTraversal methodsFor: 'enumerating' stamp: 'fbs 4/24/2011 12:38'!
collect: aBlock using: aZipper
	^ self collectNodes: [:each | aBlock value: each value ] using: aZipper! !

!DepthTraversal methodsFor: 'private' stamp: 'fbs 2/18/2011 07:28'!
depthFirstTraversal: aZipper doing: aBlock
	self subclassResponsibility.! !

!DepthTraversal methodsFor: 'enumerating' stamp: 'fbs 4/24/2011 12:40'!
traverse: aTree collecting: aBlock
	"Traverse the VALUES in aTree."
	^ self traverseNodes: aTree collecting: [:each | aBlock value: each value].! !

!DepthTraversal methodsFor: 'enumerating' stamp: 'fbs 2/21/2011 07:24'!
traverse: aTree doing: aBlock
	^ self depthFirstTraversal: aTree zipper doing: aBlock.! !

!DepthTraversal methodsFor: 'enumerating' stamp: 'fbs 4/24/2011 12:37'!
traverseNodes: aTree collecting: aBlock
	"Traverse the NODES in aTree, letting you inspect the relationship between any node and its children."
	^ self collectNodes: aBlock using: aTree zipper.! !

DepthTraversal subclass: #InOrderTraversal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!InOrderTraversal commentStamp: 'fbs 2/26/2011 23:07' prior: 0!
In-order traversal only really makes sense for binary trees: with n-ary trees, "in order" means "process the first child, then the current node, then the remaining child nodes."!

!InOrderTraversal methodsFor: 'enumerating' stamp: 'fbs 2/5/2012 22:23'!
collectNodes: aBlock using: aZipper
	"A naive depth-first traversal. It fails in the presence of cycles."
	| collected left right |
	right := #() asOrderedCollection.
	left := #() asOrderedCollection.
	collected := aZipper focus isEmpty
		ifTrue: [#() asOrderedCollection]
		ifFalse: [OrderedCollection with: (aBlock value: aZipper focus)].
		
	aZipper safeDown
		either: [:leftChild |
			left := self collectNodes: aBlock using: leftChild.
			leftChild safeRight
				either: [:rightChild |
					right := self collectNodes: aBlock using: rightChild]
				or: [:unusedError | "do nothing"]]
		or: [:unusedError | "do nothing"].

	^ left copy
		addAll: collected;
		addAll: right;
		yourself.! !

!InOrderTraversal methodsFor: 'private' stamp: 'fbs 2/25/2011 10:18'!
depthFirstTraversal: aZipper doing: aBlock
	^ (self collect: [:each | each] using: aZipper) do: aBlock.! !

DepthTraversal subclass: #PostOrderTraversal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!PostOrderTraversal commentStamp: 'fbs 2/26/2011 23:11' prior: 0!
I provide a post-order traversal of an n-tree: I traverse all child nodes before processing the current node.!

!PostOrderTraversal methodsFor: 'enumerating' stamp: 'fbs 2/5/2012 22:15'!
collectNodes: aBlock using: aZipper
	"A naive depth-first traversal. It fails in the presence of cycles."
	| collected value |
	aZipper focus isEmpty
		ifTrue: [ collected := OrderedCollection new ]
		ifFalse:
			[value := aBlock value: aZipper focus.
			collected := aZipper safeDown
				either: [:z | (self collectNodes: aBlock using: z) copy add: value; yourself]
				or: [:unusedError | { value } asOrderedCollection]].
	
	aZipper safeRight
		either: [:z |  collected addAll: (self collectNodes: aBlock using: z)]
		or: [:unusedError | "do nothing"].

	^ collected.! !

!PostOrderTraversal methodsFor: 'private' stamp: 'fbs 2/21/2011 18:01'!
depthFirstTraversal: aZipper doing: aBlock
	^ (self collect: [:each | each] using: aZipper) do: aBlock.
! !

DepthTraversal subclass: #PreOrderTraversal
	instanceVariableNames: 'root'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!PreOrderTraversal commentStamp: 'fbs 2/13/2011 17:04' prior: 0!
I provide a pre-order traversal of an n-tree: I visit the value of a node and then its subchildren.!

!PreOrderTraversal methodsFor: 'enumerating' stamp: 'fbs 2/5/2012 22:13'!
collectNodes: aBlock using: aZipper
	"A naive depth-first traversal. It fails in the presence of cycles."
	| collected value |
	aZipper focus isEmpty
		ifTrue: [ collected := OrderedCollection new ]
		ifFalse:
			[value := aBlock value: aZipper focus.
			collected := aZipper safeDown
				either: [:z | (self collectNodes: aBlock using: z) copy addFirst: value; yourself]
				or: [:unusedError | { value } asOrderedCollection]].
		
	aZipper safeRight
		either: [:z | collected addAll: (self collectNodes: aBlock using: z)]
		or: [:unusedError | "do nothing"].

	^ collected.! !

!PreOrderTraversal methodsFor: 'private' stamp: 'fbs 2/25/2011 17:37'!
depthFirstTraversal: aZipper doing: aBlock
	"A naive depth-first traversal. It fails in the presence of cycles."
	^ self collect: aBlock using: aZipper.
	
	"We're backtracking."
	"
	aZipper ifNil: [^ nil ].
	
	aBlock value: aZipper focus value.
	
	aZipper isEmpty
		ifTrue: [ ^ nil ].

	^ self depthFirstTraversal:
			(aZipper downOrElse:
				[self depthFirstTraversal:
					(aZipper rightOrElse: [
						self depthFirstTraversal:
							(aZipper up rightOrElse: [ nil ])
							doing: aBlock ])
					doing: aBlock ])
			doing: aBlock."! !

Object subclass: #ListZipper
	instanceVariableNames: 'focus trail'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!ListZipper commentStamp: 'fbs 12/28/2010 16:27' prior: 0!
A zipper is a structure that allows one, in a purely functional manner, to navigate a structure. It works by remembering the steps taken to focus on any particular part of a structure.

A ListZipper is a zipper for any sequenceable collection. It has two instance variables, namely

focus - the (sub)list on which we are currently focussed.
trail - the steps taken to focus on the focus instance variable.!

!ListZipper class methodsFor: 'instance creation' stamp: 'fbs 12/28/2010 16:48'!
on: aSequenceableCollection
	^ self new focus: aSequenceableCollection asOrderedCollection trail: OrderedCollection new.! !

!ListZipper methodsFor: 'mutating' stamp: 'fbs 12/30/2010 20:50'!
change: aBlock
	"Given an arity 1 block, replace the current value in the list with that of evaluating aBlock on that value."
	
	^ ListZipper new
		focus: (aBlock value: focus first) asOrderedCollection, focus allButFirst
		trail: trail.! !

!ListZipper methodsFor: 'accessing' stamp: 'fbs 12/28/2010 16:27'!
focus
	^ focus.! !

!ListZipper methodsFor: 'private' stamp: 'fbs 12/28/2010 16:36'!
focus: aSeqColl trail: anotherSeqColl
	focus := aSeqColl.
	trail := anotherSeqColl.! !

!ListZipper methodsFor: 'navigating' stamp: 'fbs 12/30/2010 23:22'!
left
	trail isEmpty ifTrue: [^ ZipperNavigation new signal: 'Left at the first element'].
	
	^ ListZipper new
		focus: (focus copy addFirst: trail first; yourself)
		trail: trail allButFirst.! !

!ListZipper methodsFor: 'navigating' stamp: 'fbs 12/30/2010 23:22'!
right
	focus isEmpty ifTrue: [^ ZipperNavigation new signal: 'Right at the last element'].
	
	^ ListZipper new
		focus: focus allButFirst
		trail: (trail copy addFirst: focus first; yourself)! !

Object subclass: #ZTree
	instanceVariableNames: 'children value'
	classVariableNames: 'Empty'
	poolDictionaries: ''
	category: 'Zippers-Support'!

!ZTree commentStamp: 'fbs 12/30/2010 23:32' prior: 0!
I'm a basic implementation of a tree, only present so that my package doesn't need to rely on a separate tree package.!

ZTree subclass: #MutableZTree
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!MutableZTree commentStamp: 'fbs 4/28/2011 07:52' prior: 0!
I implement a mutable version of ZTree. My sole reason for being is to provide a boundary within which mutable stuff can happen... like rebalancing a tree using the Day/Stout/Warren algorithm.!

!MutableZTree methodsFor: 'converting' stamp: 'fbs 4/30/2011 21:32'!
asImmutableTree
	"Return an immutable version of self."
	self children isEmpty ifTrue: [^ ZTree value: self value].

	^ ZTree
		value: self value
		children: (self children collect: #asImmutableTree).! !

!MutableZTree methodsFor: 'converting' stamp: 'fbs 5/1/2011 10:15'!
asImmutableTree2
	"Return an immutable version of self."
	ZTree adoptInstance: self.
	self children do: [:child | ZTree adoptInstance: child].
	^ self.! !

!MutableZTree methodsFor: 'private' stamp: 'fbs 4/28/2011 07:49'!
asTreeOfSize: size
	"Convert my vine-self to a balanced tree."
	| sz leafCount internalNodeExp |
	internalNodeExp := ((size + 1) ln / 2 ln) floor.
	sz := size.
	
	leafCount := size + 1 - (2 raisedTo: internalNodeExp).
	self compressTimes: leafCount.
	sz := sz - leafCount.
	[sz > 1] whileTrue:
		[self compressTimes: sz / 2.
		sz := sz / 2].! !

!MutableZTree methodsFor: 'private' stamp: 'fbs 4/28/2011 21:33'!
asVine
	"The first half of the DSW algorithm. Turn self into a vine, and return the number of nodes in the vine."
	| vineTail remainder tempPtr size |
	vineTail := self.
	remainder := vineTail right.
	size := 0.
	[ remainder isEmpty ] whileFalse:
		[(remainder left isEmpty)
			ifTrue: "move vineTail down one"
				[vineTail := remainder.
				remainder := remainder right.
				size := size + 1.]
			ifFalse: "rotate"
				[tempPtr := remainder left.
				remainder left: tempPtr right.
				tempPtr right: remainder.
				remainder := tempPtr.
				vineTail right: tempPtr]].
	
	^ size.! !

!MutableZTree methodsFor: 'converting' stamp: 'fbs 4/28/2011 07:49'!
balance
	"Return myself, only balanced. Use the DSW algorithm."
	| pseudoroot size |
	pseudoroot := MutableZTree new
		right: self;
		yourself.
	size := pseudoroot asVine.
	pseudoroot asTreeOfSize: size.
	^ pseudoroot right.! !

!MutableZTree methodsFor: 'accessing' stamp: 'fbs 4/27/2011 07:51'!
children
	^ children.! !

!MutableZTree methodsFor: 'private' stamp: 'fbs 4/28/2011 07:47'!
compressTimes: count
	| scanner child |
	scanner := self.
	1 to: count do:
		[:i | child := scanner right.
		scanner right: child right.
		scanner := scanner right.
		child right: scanner left.
		scanner left: child].! !

!MutableZTree methodsFor: 'initialize-release' stamp: 'fbs 4/27/2011 12:11'!
initialize
	super initialize.
	"We have 'children' rather than 'left' and 'right' so that we have the same shape as ZTree."
	children := OrderedCollection new.! !

!MutableZTree methodsFor: 'accessing' stamp: 'fbs 4/28/2011 20:57'!
left
	^ children at: 1 ifAbsent: [ZTree empty].! !

!MutableZTree methodsFor: 'accessing' stamp: 'fbs 4/27/2011 12:14'!
left: aMutableBinaryNode
	children
		ifEmpty: [children addLast: aMutableBinaryNode]
		ifNotEmpty: [children at: 1 put: aMutableBinaryNode]! !

!MutableZTree methodsFor: 'printing' stamp: 'fbs 4/27/2011 07:41'!
printString
	^ '(', self value, ' ', (self left printString), ' ', (self right printString), ')'! !

!MutableZTree methodsFor: 'accessing' stamp: 'fbs 4/28/2011 20:57'!
right
	^ children at: 2 ifAbsent: [ZTree empty].! !

!MutableZTree methodsFor: 'accessing' stamp: 'fbs 4/28/2011 20:57'!
right: aMutableBinaryNode
	"If you set right before left, you need to make sure you set left to nil before continuing."
	[children size < 2] whileTrue: [children addLast: ZTree empty].
	children at: 2 put: aMutableBinaryNode.! !

!MutableZTree methodsFor: 'accessing' stamp: 'fbs 4/26/2011 21:43'!
value
	^ value.! !

!MutableZTree methodsFor: 'accessing' stamp: 'fbs 4/26/2011 21:43'!
value: anObject
	value := anObject.! !

ZTree subclass: #ZEmptyTree
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!ZEmptyTree commentStamp: 'fbs 2/22/2011 21:41' prior: 0!
I represent an empty tree.!

!ZEmptyTree methodsFor: 'converting' stamp: 'fbs 5/1/2011 10:17'!
asImmutableTree
	^ self. ! !

!ZEmptyTree methodsFor: 'converting' stamp: 'fbs 5/1/2011 10:17'!
asImmutableTree2
	^ self. ! !

!ZEmptyTree methodsFor: 'accessing' stamp: 'fbs 2/22/2011 21:40'!
children
	^ #()! !

!ZEmptyTree methodsFor: 'testing' stamp: 'fbs 4/30/2011 17:27'!
depth
	^ 0.! !

!ZEmptyTree methodsFor: 'testing' stamp: 'fbs 4/30/2011 17:21'!
isBalanced
	^ true! !

!ZEmptyTree methodsFor: 'testing' stamp: 'fbs 2/24/2011 17:26'!
isEmpty
	^ true.! !

!ZEmptyTree methodsFor: 'printing' stamp: 'fbs 2/25/2011 17:58'!
printString
	^ '<>'.! !

!ZEmptyTree methodsFor: 'accessing' stamp: 'fbs 2/22/2011 21:40'!
value
	^ nil.! !

!ZTree class methodsFor: 'instance creation' stamp: 'fbs 2/22/2011 21:41'!
empty
	^ Empty ifNil: [Empty := ZEmptyTree new].! !

!ZTree class methodsFor: 'instance creation' stamp: 'fbs 2/25/2011 17:53'!
value: anObject
	^ anObject = ZTree empty
		ifTrue: [ ZTree empty ]
		ifFalse: [ self new
			value: anObject
			children: OrderedCollection new ]! !

!ZTree class methodsFor: 'instance creation' stamp: 'fbs 12/30/2010 23:41'!
value: anObject children: aCollection
	^ self new value: anObject children: aCollection.! !

!ZTree methodsFor: 'comparing' stamp: 'fbs 4/27/2011 17:49'!
= anObject
	^ anObject class == self class and:
		[self value = anObject value and:
			[self childrenEqual: anObject children]].! !

!ZTree methodsFor: 'printing' stamp: 'fbs 4/24/2011 12:33'!
asDotFile
	| join |
	join := [:c | c
			ifEmpty: ['']
			ifNotEmpty: [ c reduce: [:s :e | s, ';', Character cr, e ]]].

	^ 'digraph {', Character cr,
		self printChildRecords, Character cr,
		(join value: (self associations collect: #printString)), Character cr,
		'}'! !

!ZTree methodsFor: 'printing' stamp: 'fbs 4/24/2011 20:56'!
asDotNode
	| labels labelString |
	labels := OrderedCollection new.
	(1 to: self children size do: [:i | labels addLast: '<p', i printString, '>']).
	labelString := labels ifEmpty: ['']
		ifNotEmpty: ['[label = "', (labels reduce: [:acc :each | acc , ' | ']), '"]'].
	^ '"node', self value printString, labelString, ';'! !

!ZTree methodsFor: 'converting' stamp: 'fbs 4/28/2011 21:21'!
asMutableTree
	"Return a mutable copy of this tree."
	| mutableCopy |
	self isEmpty ifTrue: [^ZTree empty].

	mutableCopy := MutableZTree new.
	mutableCopy value: self value.
	mutableCopy children addAll: (self children collect: #asMutableTree).
	^ mutableCopy.! !

!ZTree methodsFor: 'converting' stamp: 'fbs 4/30/2011 22:30'!
asOrderedCollection
	^ self collect: #yourself! !

!ZTree methodsFor: 'private' stamp: 'fbs 4/24/2011 10:35'!
associations
	^ self associationsWithLabels: [:node | node value].! !

!ZTree methodsFor: 'private' stamp: 'fbs 4/23/2011 22:00'!
associationsWithLabels: aBlock
	^ (self children
		inject: OrderedCollection new
		into: [:acc :each | acc addAll: each associations; yourself]) ,
	(self children collect: [:child | (aBlock value: self) -> (aBlock value: child)]).! !

!ZTree methodsFor: 'converting' stamp: 'fbs 4/30/2011 17:09'!
balance
	"Return a balanced copy of self."
	^ self asOrderedCollection asBalancedBinaryTree.! !

!ZTree methodsFor: 'converting' stamp: 'fbs 4/30/2011 21:18'!
balanceDSW
	"Return a balanced copy of self."
	^ self asMutableTree balance asImmutableTree.! !

!ZTree methodsFor: 'converting' stamp: 'fbs 5/1/2011 10:16'!
balanceDSW2
	"Return a balanced copy of self."
	^ self asMutableTree balance asImmutableTree2.! !

!ZTree methodsFor: 'accessing' stamp: 'fbs 12/30/2010 23:33'!
children
	^ children! !

!ZTree methodsFor: 'private' stamp: 'fbs 4/27/2011 17:54'!
childrenEqual: aCollection
	"Compare this node's children against another collection of children. Note that a missing child is semantically identical to that child being the empty (sub)tree."
	1 to: (children size max: aCollection size) do:
		[:i | | left right |
		left := children at: i ifAbsent: [ZTree empty].
		right := aCollection at: i ifAbsent: [ZTree empty].
		(left = right) ifFalse: [^false]].
	^ true.! !

!ZTree methodsFor: 'enumerating' stamp: 'fbs 4/30/2011 22:29'!
collect: aBlock
	^ InOrderTraversal new traverse: self collecting: aBlock.! !

!ZTree methodsFor: 'testing' stamp: 'fbs 4/30/2011 17:28'!
depth
	self hasChildren ifFalse: [ ^ 1 ].
	
	^ 1 + (children collect: #depth) max.! !

!ZTree methodsFor: 'testing' stamp: 'fbs 4/30/2011 15:32'!
hasChildren
	^ (children reject: #isEmpty) isEmpty not.! !

!ZTree methodsFor: 'initialize-release' stamp: 'fbs 12/30/2010 23:34'!
initialize
	super initialize.
	children := OrderedCollection new.! !

!ZTree methodsFor: 'testing' stamp: 'fbs 4/30/2011 17:31'!
isBalanced
	| depths |
	self hasChildren ifFalse: [ ^ true ].

	depths := (self children collect: #depth) asSortedCollection.
	"A tree with one subtree of depth 1is balanced."
	(depths size = 1) ifTrue: [ ^ depths first = 1 ].
	"A tree with many subtrees is balanced if there is at most a difference of 1 between the depths of its subtrees."
	^ (depths last - depths first) <= 1.! !

!ZTree methodsFor: 'testing' stamp: 'fbs 2/24/2011 17:26'!
isEmpty
	^ false.! !

!ZTree methodsFor: 'private' stamp: 'fbs 4/24/2011 12:48'!
printChildRecords
	^ (PreOrderTraversal new
		traverseNodes: self
		collecting: #asDotNode)
		reduce: [:acc :e | acc , Character cr, e].! !

!ZTree methodsFor: 'printing' stamp: 'fbs 12/30/2010 23:54'!
printString
	^ '(', self value printString,
		(children
			ifEmpty: ['']
			ifNotEmpty: [' ', (self children reduce: [:a :b | a printString, ', ', b printString])]),
		')'.! !

!ZTree methodsFor: 'accessing' stamp: 'fbs 12/31/2010 16:54'!
value
	^ value.! !

!ZTree methodsFor: 'private' stamp: 'fbs 12/31/2010 16:54'!
value: anObject children: aCollection
	value := anObject.
	children := aCollection asOrderedCollection.! !

!ZTree methodsFor: 'enumerating' stamp: 'fbs 12/31/2010 19:21'!
zipper
	^ TreeZipper on: self.! !

Object subclass: #ZipperContext
	instanceVariableNames: 'leftNodes rightNodes path changed visitedNodes'
	classVariableNames: 'Root'
	poolDictionaries: ''
	category: 'Zippers'!

!ZipperContext commentStamp: 'fbs 12/20/2011 20:42' prior: 0!
I am a recursive structure that describes a path through a tree.

  self path

 refers to the previous position in the path, itself a TreeZipperNode. The first entry in a path has

  self path == nil

Being part of a zipper, I naturally represent a context-with-a-hole - that part of a tree minus the part currently under focus.

leftNodes: my siblings to the left (or, those siblings processed before me in a pre-order traversal)
rightNodes: my siblings to the right
parentNodes: an array containing the reverse path needed to reach the original node visited by my zipper.
changed: true if I represent a changed context - the addition or removal of a sibling, etc.!

ZipperContext subclass: #EndOfTraversalContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Support'!

!EndOfTraversalContext commentStamp: 'fbs 2/4/2012 22:10' prior: 0!
A context representing the end of a traversal.!

!EndOfTraversalContext class methodsFor: 'instance creation' stamp: 'fbs 2/4/2012 23:27'!
trail: aZipperContext
	^ self
		path: aZipperContext path
		leftNodes: aZipperContext leftNodes
		rightNodes: aZipperContext rightNodes
		parentNodes: aZipperContext parentNodes
		changed: aZipperContext changed.! !

!EndOfTraversalContext methodsFor: 'testing' stamp: 'fbs 2/4/2012 22:29'!
isEnd
	^ true.! !

ZipperContext subclass: #RootContext
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!RootContext commentStamp: 'fbs 2/4/2012 22:10' prior: 0!
A context representing the start of a traversal.!

!RootContext methodsFor: 'accessing' stamp: 'fbs 10/18/2011 16:27'!
changed
	^ false.! !

!RootContext methodsFor: 'testing' stamp: 'fbs 2/4/2012 22:28'!
isRoot
	^ true.! !

!RootContext methodsFor: 'accessing' stamp: 'fbs 10/31/2011 22:01'!
leftNodes
	^ #()! !

!RootContext methodsFor: 'accessing' stamp: 'fbs 12/20/2011 20:39'!
parentNodes
	^ #().! !

!RootContext methodsFor: 'accessing' stamp: 'fbs 11/1/2011 07:59'!
path
	"This lets us avoid unsightly conditionals in the zipper: the path of no path is no path. Zipping up isn't a problem because #safeUp checks #isEnd."
	^ self.! !

!RootContext methodsFor: 'accessing' stamp: 'fbs 10/31/2011 22:00'!
rightNodes
	^ #()! !

!ZipperContext class methodsFor: 'class initialization' stamp: 'fbs 2/4/2012 22:43'!
initialize
	Root := RootContext new.! !

!ZipperContext class methodsFor: 'instance creation' stamp: 'fbs 12/20/2011 20:37'!
path: aTreeZipperTrail leftNodes: leftCollection rightNodes: rightCollection parentNodes: someObjects changed: aBoolean
	^ self new
		path: aTreeZipperTrail
		leftNodes: leftCollection
		rightNodes: rightCollection
		parentNodes: someObjects
		changed: aBoolean.! !

!ZipperContext class methodsFor: 'instance creation' stamp: 'fbs 2/4/2012 22:43'!
root
	^ Root.! !

!ZipperContext methodsFor: 'accessing' stamp: 'fbs 10/17/2011 17:52'!
changed
	^ changed.! !

!ZipperContext methodsFor: 'testing' stamp: 'fbs 10/18/2011 16:20'!
isEnd
	^ false.! !

!ZipperContext methodsFor: 'testing' stamp: 'fbs 2/4/2012 22:38'!
isRoot
	^ false.! !

!ZipperContext methodsFor: 'accessing' stamp: 'fbs 12/31/2010 00:19'!
leftNodes
	^ leftNodes! !

!ZipperContext methodsFor: 'accessing' stamp: 'fbs 12/20/2011 12:22'!
parentNode
	^ visitedNodes last.! !

!ZipperContext methodsFor: 'accessing' stamp: 'fbs 12/20/2011 20:38'!
parentNodes
	^ visitedNodes! !

!ZipperContext methodsFor: 'accessing' stamp: 'fbs 12/31/2010 16:22'!
path
	^ path.! !

!ZipperContext methodsFor: 'private' stamp: 'fbs 12/20/2011 20:36'!
path: aTreeZipperTrail leftNodes: leftCollection rightNodes: rightCollection parentNodes: someObjects changed: aBoolean
	path := aTreeZipperTrail.
	leftNodes := leftCollection.
	rightNodes := rightCollection.
	visitedNodes := someObjects.
	changed := aBoolean.! !

!ZipperContext methodsFor: 'accessing' stamp: 'fbs 12/31/2010 00:19'!
rightNodes
	^ rightNodes.! !

Object subclass: #ZipperError
	instanceVariableNames: 'error location'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!ZipperError class methodsFor: 'instance creation' stamp: 'fbs 2/5/2012 08:18'!
error: aSymbol location: aZipper
	^ self new error: aSymbol location: aZipper.! !

!ZipperError methodsFor: 'accessing' stamp: 'fbs 2/5/2012 08:18'!
error
	^ error.! !

!ZipperError methodsFor: 'private' stamp: 'fbs 2/5/2012 08:19'!
error: aSymbol location: aZipper
	error := aSymbol.
	location := aZipper.! !

!ZipperError methodsFor: 'accessing' stamp: 'fbs 2/5/2012 08:18'!
location
	^ location.! !

Error subclass: #ZipperNavigation
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers'!

!ZipperNavigation commentStamp: 'fbs 12/28/2010 17:36' prior: 0!
Raise me whenever a Zipper tries to navigate to somewhere it cannot - past the end of a list, to a missing subtree, etc.!

!SequenceableCollection methodsFor: '*Zippers' stamp: 'fbs 4/29/2011 09:52'!
asBalancedBinaryTree
	^ self asBalancedBinaryTreeFrom: 1 to: self size.! !

!SequenceableCollection methodsFor: '*Zippers' stamp: 'fbs 4/30/2011 17:11'!
asBalancedBinaryTreeFrom: startIdx to: endIdx
	| mid |
	(startIdx > endIdx) ifTrue: [ ^ ZTree empty ].
	(startIdx = endIdx) ifTrue: [ ^ ZTree value: (self at: startIdx) ].
	
	mid := ((startIdx + endIdx) / 2) floor.
	^ ZTree value: (self at: mid)
		children:
			{self asBalancedBinaryTreeFrom: startIdx to: mid - 1.
			self asBalancedBinaryTreeFrom: mid + 1 to: endIdx.}.! !

TestCase subclass: #BinarySearchTreeTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!BinarySearchTreeTest methodsFor: 'testing - adding' stamp: 'fbs 5/1/2011 14:01'!
testAddToEmptyTreeAdds
	| items tree |
	tree := (BinarySearchTree tree: ZTree empty) add: 1.
	items := InOrderTraversal new traverse: tree collecting: #yourself.
	
	self assert: #(1) asOrderedCollection = items.! !

!BinarySearchTreeTest methodsFor: 'testing - adding' stamp: 'fbs 5/1/2011 14:00'!
testAddingLargerPreservesOrder
	"The in-order traversal of a binary tree always yields a sorted collection."
	| items tree |
	tree := (BinarySearchTree with: 2) add: 3.
	items := InOrderTraversal new traverse: tree collecting: #yourself.
	
	self assert: #(2 3) asOrderedCollection = items.! !

!BinarySearchTreeTest methodsFor: 'testing - adding' stamp: 'fbs 5/1/2011 14:00'!
testAddingPreservesOrder
	"The in-order traversal of a binary tree always yields a sorted collection."
	| items tree |
	tree := ((BinarySearchTree with: 1) add: 3) add: 2.
	items := InOrderTraversal new traverse: tree collecting: #yourself.
	
	self assert: #(1 2 3) asOrderedCollection = items.! !

!BinarySearchTreeTest methodsFor: 'testing - adding' stamp: 'fbs 5/1/2011 14:00'!
testAddingSmallerPreservesOrder
	"The in-order traversal of a binary tree always yields a sorted collection."
	| items tree |
	tree := (BinarySearchTree with: 2) add: 1.
	items := InOrderTraversal new traverse: tree collecting: #yourself.
	
	self assert: #(1 2) asOrderedCollection = items.! !

!BinarySearchTreeTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/26/2011 22:50'!
testCollectExecutesBlockOncePerNode
	| items t |
	t := ((BinarySearchTree with: 2) add: 3) add: 1.
	items := t collect: [:each | each * 2].
	self assert: items = (#(1 2 3) * 2) asOrderedCollection.! !

!BinarySearchTreeTest methodsFor: 'testing - adding' stamp: 'fbs 2/24/2011 17:32'!
testNewTreeIsBlank
	self assert: BinarySearchTree new asOrderedCollection = #() asOrderedCollection.! !

!BinarySearchTreeTest methodsFor: 'testing - converting' stamp: 'fbs 4/30/2011 17:52'!
testRebalance
	| bst zipper |
	bst := BinarySearchTree tree: ZTree empty.
	"Construct a degenerate BST."
	1 to: 10 do: [:i | bst := bst add: i].
	
	zipper := bst rebalanced zipper.
	self assert: zipper root isBalanced.
	self assert: (1 to: 10) asOrderedCollection = (InOrderTraversal new collect: #yourself using: zipper) asOrderedCollection.! !

!BinarySearchTreeTest methodsFor: 'testing - instance creation' stamp: 'fbs 2/24/2011 22:52'!
testWithReturnsSingleNodeTree
	| bst value |
	value := 1.
	bst := BinarySearchTree with: value.
	self assert: (bst collect: [:each | each]) = #(1) asOrderedCollection.
	! !

TestCase subclass: #ListZipperTest
	instanceVariableNames: 'prefix suffix'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!ListZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/30/2010 21:13'!
testChangeChangesValue
	| list zipper |
	list := #(1 2 3) asOrderedCollection.
	zipper := ListZipper on: list.
	zipper := zipper right right.
	zipper := zipper change: [:x | x + 1].
	
	zipper := zipper left left.
	
	self assert: zipper focus = #(1 2 4) asOrderedCollection.! !

!ListZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/30/2010 21:13'!
testChangePreservesOriginal
	| original list zipper |
	original := list := #(1 (2 3) 4) asOrderedCollection.
	zipper := ListZipper on: list.
	zipper := zipper right.
	zipper := zipper change: [:x | x size].
	zipper := zipper left.
	
	self assert: zipper focus = #(1 2 4) asOrderedCollection.
	self assert: list = original description: 'Zipper not functional'.! !

!ListZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/30/2010 21:15'!
testChangeSharesStructure
	| original list zipper |
	original := list := #(1 (2 3) 4) asOrderedCollection.
	zipper := ListZipper on: list.
	zipper := zipper right right.
	zipper := zipper change: [:x | x + 1].
	zipper := zipper left left.
	
	self assert: zipper focus = #(1 (2 3) 5) asOrderedCollection.
	self assert: zipper focus second == original second description: 'Zipper not sharing original structure'.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/28/2010 16:50'!
testLeft
	| left list zipper right |
	list := #(1 2 3) asOrderedCollection.
	zipper := ListZipper on: list.
	right := zipper right right.
	
	left := right left.
	self assert: left focus = list allButFirst.
	
	left := left left.
	self assert: left focus = list.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:17'!
testLeftPastBeginningOfCollectionSignalsException
	| zipper |
	zipper := ListZipper on: #(1 2).
	self
		should: [ zipper right left left ]
		raise: ZipperNavigation.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/28/2010 16:49'!
testOn
	| list zipper |
	list := #(1 2 3 4 5).
	zipper := ListZipper on: list.
	self assert: zipper focus = list asOrderedCollection.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/28/2010 16:50'!
testRight
	| list zipper right |
	list := #(1 2 3) asOrderedCollection.
	zipper := ListZipper on: list.
	right := zipper right.
	self assert: right focus = list allButFirst.
	
	right := right right.
	self assert: right focus = list allButFirst allButFirst.! !

!ListZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:17'!
testRightPastEndOfCollectionSignalsException
	| zipper |
	zipper := ListZipper on: #(1 2).
	self
		should: [ zipper right right right ]
		raise: ZipperNavigation.! !

TestCase subclass: #OrderedCollectionZipperSupportTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!OrderedCollectionZipperSupportTest methodsFor: 'testing - converting' stamp: 'fbs 4/28/2011 23:20'!
testEmptyCollectionAsBalancedBinaryTreeIsEmptyTree
	self assert: #() asBalancedBinaryTree == ZTree empty.! !

!OrderedCollectionZipperSupportTest methodsFor: 'testing - converting' stamp: 'fbs 4/30/2011 17:13'!
testEvenCollectionAsBalancedBinaryTreeIsEmptyTree
	| expected |
	expected := ZTree
		value: 2
		children: {ZTree value: 1.
		ZTree value: 3 children: {ZTree empty. ZTree value: 4}}.
	self assert: #(1 2 3 4) asBalancedBinaryTree = expected.! !

!OrderedCollectionZipperSupportTest methodsFor: 'testing - converting' stamp: 'fbs 4/30/2011 17:13'!
testOddCollectionAsBalancedBinaryTreeIsEmptyTree
	| expected |
	expected := ZTree
		value: 2
		children: {ZTree value: 1. ZTree value: 3}.
	self assert: #(1 2 3) asBalancedBinaryTree = expected.! !

!OrderedCollectionZipperSupportTest methodsFor: 'testing - converting' stamp: 'fbs 4/30/2011 17:13'!
testSingleCollectionAsBalancedBinaryTreeIsSingleNode
	self assert: #(0) asBalancedBinaryTree = (ZTree value: 0).! !

TestCase subclass: #TraversalTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

TraversalTestCase subclass: #BreadthFirstSearchTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!BreadthFirstSearchTest methodsFor: 'running' stamp: 'fbs 2/9/2012 09:22'!
makeTraversal
	^ BreadthFirstSearch new.! !

TraversalTestCase subclass: #InOrderTraversalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!InOrderTraversalTest methodsFor: 'running' stamp: 'fbs 2/24/2011 17:37'!
makeTraversal
	^ InOrderTraversal new.! !

!InOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/28/2011 17:46'!
testCollectEnumeratesCorrectly
	| i tree |
	tree := ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5}.
	i := self makeTraversal traverse: tree collecting: [:each | each].
	
	self assert: i asOrderedCollection = #(3 2 4 1 5) asOrderedCollection.! !

!InOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 4/24/2011 22:04'!
testTraversalHandlesEmptySubtrees
	| i tree |
	tree := ZTree value: 1 children: {ZTree empty. ZTree value: 2}.
	i := self makeTraversal traverseNodes: tree collecting: #value.
	
	self assert: i asOrderedCollection = #(1 2) asOrderedCollection.! !

TraversalTestCase subclass: #PostOrderTraversalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!PostOrderTraversalTest methodsFor: 'running' stamp: 'fbs 2/18/2011 07:24'!
makeTraversal
	^ PostOrderTraversal new.! !

!PostOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/28/2011 17:47'!
testCollectCollectsCorrectly
	| i tree |
	tree := ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5}.
	i := self makeTraversal traverse: tree collecting: [:each | each * 2].
	
	self assert: i asOrderedCollection = (#(3 4 2 5 1) * 2) asOrderedCollection.! !

!PostOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/28/2011 17:47'!
testCollectEnumeratesCorrectly
	| i tree |
	tree := ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5}.
	i := self makeTraversal traverse: tree collecting: [:each | each].
	
	self assert: i asOrderedCollection = #(3 4 2 5 1) asOrderedCollection.! !

!PostOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 4/24/2011 21:30'!
testCollectNodesEnumeratesCorrectly
	| i tree |
	tree := ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5}.
	i := self makeTraversal traverseNodes: tree collecting: #value.
	
	self assert: i asOrderedCollection = #(3 4 2 5 1) asOrderedCollection.! !

!PostOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 4/24/2011 21:35'!
testTraversalHandlesEmptySubtrees
	| i tree |
	tree := ZTree value: 1 children: {ZTree empty. ZTree value: 2}.
	i := self makeTraversal traverseNodes: tree collecting: #value.
	
	self assert: i asOrderedCollection = #(2 1) asOrderedCollection.! !

!PostOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/18/2011 07:26'!
testTreeTraversalMovesDepthFirst
	self
		assertTraversalOf: (ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5})
		 yieldsOrder: #(3 4 2 5 1).! !

!PostOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/18/2011 07:51'!
testTreeTraversalMovesDepthFirstOnTriangleTree
	self
		assertTraversalOf: (ZTree value: 1 children: {ZTree value: 2. ZTree value: 3})
		 yieldsOrder: #(2 3 1).! !

TraversalTestCase subclass: #PreOrderTraversalTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!PreOrderTraversalTest methodsFor: 'running' stamp: 'fbs 2/17/2011 22:04'!
makeTraversal
	^ PreOrderTraversal new.! !

!PreOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/28/2011 17:47'!
testCollectCollectsCorrectly
	| i tree |
	tree := ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5}.
	i := self makeTraversal traverse: tree collecting: [:each | each * 2].
	
	self assert: i asOrderedCollection = (#(1 2 3 4 5) * 2) asOrderedCollection.! !

!PreOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 4/24/2011 21:56'!
testTraversalHandlesEmptySubtrees
	| i tree |
	tree := ZTree value: 1 children: {ZTree empty. ZTree value: 2}.
	i := self makeTraversal traverseNodes: tree collecting: #value.
	
	self assert: i asOrderedCollection = #(1 2) asOrderedCollection.! !

!PreOrderTraversalTest methodsFor: 'testing - enumerating' stamp: 'fbs 2/17/2011 22:11'!
testTreeTraversalMovesDepthFirst
	self
		assertTraversalOf: (ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5})
		 yieldsOrder: #(1 2 3 4 5).! !

!TraversalTestCase class methodsFor: 'testing' stamp: 'fbs 2/26/2011 22:55'!
isAbstract
	^ self name = #TraversalTestCase.! !

!TraversalTestCase methodsFor: 'testing' stamp: 'fbs 2/17/2011 22:08'!
assertTraversalOf: aTree yieldsOrder: aCollection
	| i |
	i := #() asOrderedCollection.
	self makeTraversal traverse: aTree doing: [:each | i add: each ].
	
	self assert: i = aCollection asOrderedCollection.! !

!TraversalTestCase methodsFor: 'running' stamp: 'fbs 2/17/2011 22:04'!
makeTraversal
	^ self subclassResponsibility.! !

!TraversalTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 2/17/2011 22:05'!
testEmptyTraversalDoesNothing
	| i |
	i := 0.
	self makeTraversal traverse: (ZTree value: 1) doing: [:each | i := i + 1].
	
	self assert: i = 1.! !

!TraversalTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 2/28/2011 17:45'!
testEnumeratingEmptyTreeReturnsEmptyCollection
	| items |
	items := self makeTraversal traverse: ZTree empty collecting: [:each | each ].
	self assert: items isEmpty.! !

!TraversalTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 2/17/2011 22:05'!
testTraverseBranchingTreeCallsBlock
	| i |
	i := 0.
	self makeTraversal traverse: (ZTree value: 0 children: {ZTree value: 1. ZTree value: 2}) doing: [:each | i := i + 1].
	
	self assert: i = 3.! !

!TraversalTestCase methodsFor: 'testing - enumerating' stamp: 'fbs 2/17/2011 22:06'!
testTreeTraversalCallsBlock
	| i |
	i := 0.
	self makeTraversal traverse: (ZTree value: 1) doing: [:each | i := i + 1].
	
	self assert: i = 1.! !

TestCase subclass: #TreeTestCases
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

TreeTestCases subclass: #BinaryTreeZipperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!BinaryTreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 4/30/2011 09:40'!
makeZipperOn: aTree
	^ BinaryTreeZipper on: aTree.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 4/30/2011 16:10'!
testCopy
	| copy expected received zipper |
	zipper := BinaryTreeZipper on: self triangleTree.
	copy := zipper copy.
	expected := PreOrderTraversal new collect: #yourself using: zipper.
	received := PreOrderTraversal new collect: #yourself using: copy.
	self assert: expected = received.! !

!BinaryTreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/20/2011 16:30'!
testInsertDownInsertsLeftSubtree
	| tree zipper |
	tree := ZTree value: 0.
	zipper := BinaryTreeZipper on: tree.
	
	zipper := zipper insertDown: (ZTree value: 1).
	self assert: zipper focus = (ZTree value: 1) description: 'Zipper not focused on new node'.
	self shouldnt: [zipper up] raise: ZipperNavigation.
	
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 1}).! !

!BinaryTreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/20/2011 16:39'!
testInsertDownOnEmptyTreeIsATreeWithOneNode
	| tree zipper |
	tree := ZTree empty.
	zipper := BinaryTreeZipper on: tree.
	
	zipper := zipper insertDown: (ZTree value: 0).
	self assert: zipper focus value = 0 description: 'Zipper not focused on new node'.
	
	self assert: zipper root = (ZTree value: 0).! !

!BinaryTreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/20/2011 20:30'!
testInsertLeftInsertsLeftSubtree
	| tree zipper |
	tree := ZTree value: 0.
	zipper := BinaryTreeZipper on: tree.
	
	zipper := zipper insertLeft: (ZTree value: 1).
	self assert: zipper focus value = 1 description: 'Zipper not focused on new node'.
	self shouldnt: [zipper up] raise: ZipperNavigation.
	
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 1}).! !

!BinaryTreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/20/2011 16:31'!
testInsertLeftOnEmptyTreeIsATreeWithOneNode
	| tree zipper |
	tree := ZTree empty.
	zipper := BinaryTreeZipper on: tree.
	
	zipper := zipper insertDown: (ZTree value: 0).
	self assert: zipper focus value = 0 description: 'Zipper not focused on new node'.
	
	self assert: zipper root = (ZTree value: 0).! !

!BinaryTreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 4/30/2011 17:07'!
testInsertRightInsertsRightSubtree
	| tree zipper |
	tree := ZTree value: 0.
	zipper := BinaryTreeZipper on: tree.
	
	zipper := zipper insertRight: 1.
	self assert: zipper focus value = 1 description: 'Zipper not focused on new node'.
	self shouldnt: [zipper up] raise: ZipperNavigation.
	self assert: zipper root = (ZTree value: 0 children: {ZTree empty. ZTree value: 1}).! !

!BinaryTreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/20/2011 16:39'!
testInsertRightOnEmptyTreeIsATreeWithOneNode
	| tree zipper |
	tree := ZTree empty.
	zipper := BinaryTreeZipper on: tree.
	
	zipper := zipper insertRight: (ZTree value: 0).
	self assert: zipper focus value = 0 description: 'Zipper not focused on new node'.
	
	self assert: zipper root = (ZTree value: 0).! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 4/30/2011 15:55'!
testLeftMovesToLeftChild
	| tree zipper |
	tree := self triangleTree.
	zipper := BinaryTreeZipper on: tree.
	zipper := zipper left.
	
	self assert: zipper focus value = 2 description: 'We followed the wrong path'.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 4/30/2011 14:15'!
testLeftOnEmptyTreeFails
	| z |
	z := self makeZipperOn: self emptyTree.
	self should: [ z left ] raise: ZipperNavigation.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 4/30/2011 15:27'!
testLeftOnNoLeftSubtreeFails
	| z |
	z := self makeZipperOn: (ZTree value: 0).
	self should: [ z left ] raise: ZipperNavigation.! !

!BinaryTreeZipperTest methodsFor: 'testing - instance creation' stamp: 'fbs 4/30/2011 14:13'!
testOn
	| tree zipper |
	tree := self emptyTree.
	zipper := BinaryTreeZipper on: tree.
	
	self assert: zipper focus = tree.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 4/30/2011 14:32'!
testRightMovesToRightSibling
	| tree zipper |
	tree := self triangleTree.
	zipper := BinaryTreeZipper on: tree.
	zipper := zipper right.
	
	self assert: zipper focus value = 3 description: 'We followed the wrong path'.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 4/30/2011 14:29'!
testRightOnEmptyTreeFails
	| z |
	z := self makeZipperOn: self emptyTree.
	self should: [ z right ] raise: ZipperNavigation.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 4/30/2011 17:12'!
testRightOnNoRightSubtreeFails
	| z |
	z := self makeZipperOn: (ZTree value: 0).
	self should: [ z right ] raise: ZipperNavigation.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 4/30/2011 15:26'!
testRightWithNoLeftSubtreeMovesToRightSubtree
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree empty. ZTree value: 1}.
	zipper := BinaryTreeZipper on: tree.
	zipper := zipper right.
	
	self assert: zipper focus value = 1 description: 'We followed the wrong path'.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/5/2012 22:34'!
testSafeLeftWithEmptyLeftSubtreeFails
	| move tree zipper |
	tree := ZTree value: 0 children: {ZTree empty}.
	zipper := BinaryTreeZipper on: tree.
	move := zipper safeLeft.
	self deny: move isRight.
	self assert: #noLeftSubtree equals: move value error.
	self assert: move value location focus value = 0 description: 'Focus moved'.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/5/2012 22:34'!
testSafeLeftWithNoLeftSubtreeFails
	| move tree zipper |
	tree := ZTree value: 0.
	zipper := BinaryTreeZipper on: tree.
	move := zipper safeLeft.
	self assert: #noLeftSubtree equals: move value error.
	self assert: move value location focus value = 0 description: 'Focus moved'.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/5/2012 22:31'!
testSafeRightWithEmptyRightSubtreeFails
	| move tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1. ZTree empty}.
	zipper := BinaryTreeZipper on: tree.
	move := zipper safeRight.
	self assert: #noRightSubtree equals: move value error.
	self assert: move value location focus value = 0 description: 'Focus moved'.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/5/2012 22:30'!
testSafeRightWithNoLeftSubtreeMovesToRightSubtree
	| move tree zipper |
	tree := ZTree value: 0 children: {ZTree empty. ZTree value: 1}.
	zipper := BinaryTreeZipper on: tree.
	move := zipper safeRight.
	self assert: move isRight.
	self assert: move value focus value = 1 description: 'We followed the wrong path'.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/5/2012 22:30'!
testSafeRightWithNoRightSubtreeFails
	| move tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1}.
	zipper := BinaryTreeZipper on: tree.
	move := zipper safeRight.
	self deny: move isRight.
	self assert: #noRightSubtree equals: move value error.
	self assert: move value location focus value = 0 description: 'Focus moved'.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/5/2012 22:29'!
testSafeRightWithNoSubtreesFails
	| move tree zipper |
	tree := ZTree value: 0.
	zipper := BinaryTreeZipper on: tree.
	move := zipper safeRight.
	self deny: move isRight.
	self assert: #rightAtLeafNode equals: move value error.
	self assert: move value location focus value = 0 description: 'Focus moved'.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/5/2012 22:24'!
testSafeUpOnLeftSubtreeMovesToParent
	| move zipper |
	zipper := BinaryTreeZipper on: self triangleTree.
	move := zipper left safeUp.
	
	self assert: move isRight.
	self assert: move value focus value = 1.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/5/2012 22:24'!
testSafeUpOnRightSubtreeMovesToParent
	| move zipper |
	zipper := BinaryTreeZipper on: self triangleTree.
	move := zipper right safeUp.
	
	self assert: move isRight.
	self assert: move value focus value = 1.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/5/2012 22:23'!
testSafeUpOnRootNodeFails
	| move zipper |
	zipper := BinaryTreeZipper on: (ZTree value: 0).
	move := zipper safeUp.
	self deny: move isRight.
	self assert: #upAtRootNode equals: move value error.
	self assert: move value location focus value = 0.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 4/30/2011 16:01'!
testUpOnEmptyTreeFails
	| zipper |
	zipper := BinaryTreeZipper on: ZTree empty.
	self should: [zipper up] raise: ZipperNavigation.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 4/30/2011 16:03'!
testUpOnLeftSubtreeMovesToParent
	| zipper |
	zipper := BinaryTreeZipper on: self triangleTree.
	zipper := zipper left up.
	
	self assert: zipper focus value = 1.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 4/30/2011 16:03'!
testUpOnRightSubtreeMovesToParent
	| zipper |
	zipper := BinaryTreeZipper on: self triangleTree.
	zipper := zipper right up.
	
	self assert: zipper focus value = 1.! !

!BinaryTreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 4/30/2011 16:04'!
testUpOnRootNodeFails
	| zipper |
	zipper := BinaryTreeZipper on: (ZTree value: 0).
	self should: [zipper up] raise: ZipperNavigation.! !

!TreeTestCases class methodsFor: 'testing' stamp: 'fbs 2/26/2011 22:55'!
isAbstract
	^ self name = #TreeTestCases.! !

!TreeTestCases methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 17:35'!
emptyTree
	^ ZTree empty.! !

!TreeTestCases methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 08:00'!
flatTree
	" Best viewed in a monospaced font:
	       1
	   / / | \ \
	  2 3  4 5 6"
	^ ZTree value: 1 children: ((2 to: 6) collect: [ :i | ZTree value: i])! !

!TreeTestCases methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 08:00'!
nestedTriangleTree
	" Best viewed in a monospaced font:
	    1
	   / \
	  2   3 
	     / \
	    4   5 "
	| b c |
	b := ZTree value: 2.
	c := ZTree value: 3 children: {ZTree value: 4. ZTree value: 5}.
	^ ZTree value: 1 children: {b. c.}! !

!TreeTestCases methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 07:59'!
triangleTree
	" Best viewed in a monospaced font:
	   1
	  / \
	 2   3 "
	| b c |
	b := ZTree value: 2.
	c := ZTree value: 3.
	^ ZTree value: 1 children: {b. c.}! !

TreeTestCases subclass: #TreeZipperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

TreeZipperTest subclass: #PluggableZipperTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!PluggableZipperTest methodsFor: 'fixtures' stamp: 'fbs 2/6/2012 19:08'!
makeZipperOn: aTree
	"These are copies of TreeZipper's 'pluggable behaviour' category methods."
	^ PluggableZipper
		on: aTree 
		childrenBlock: [:node | node children]
		branchBlock: [:node | node children notEmpty]
		newNodeBlock: [:anObject :someObjects |
			(anObject isKindOf: ZTree)
				ifTrue: [ZTree value: anObject value children: someObjects]
				ifFalse: [ZTree value: anObject children: someObjects]].! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 17:36'!
emptyTree
	^ ZTree empty.! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 12/31/2010 18:21'!
flatTree
	" Best viewed in a monospaced font:
	       1
	   / / | \ \
	  2 3  4 5 6"
	^ ZTree value: 1 children: ((2 to: 6) collect: [ :i | ZTree value: i])! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 2/25/2011 08:05'!
makeZipperOn: aTree
	^ TreeZipper on: aTree.! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 12/31/2010 17:58'!
nestedTriangleTree
	" Best viewed in a monospaced font:
	    1
	   / \
	  2   3 
	     / \
	    4   5 "
	| b c |
	b := ZTree value: 2.
	c := ZTree value: 3 children: {ZTree value: 4. ZTree value: 5}.
	^ ZTree value: 1 children: {b. c.}! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/20/2011 20:30'!
testChangeReplacesValue
	| tree zipper |
	tree := self triangleTree.
	zipper := (TreeZipper on: tree) down.
	
	zipper := zipper change: [ :z | ZTree value: (z value + 1)].
	self assert: zipper focus value = 3 description: 'No change made'.
	self assert: zipper right focus value = 3 description: 'Position changed'.

	self
		assert: (ZTree value: 1 children: {ZTree value: 3. ZTree value: 3.}) printString
		equals: zipper root printString.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 2/5/2012 21:36'!
testChangeToAtTopReplacesValue
	| tree zipper |
	tree := self triangleTree.
	zipper := TreeZipper on: tree.
	
	zipper := zipper changeTo: (ZTree value: 5 children: zipper focus children).
	self assert: zipper focus value = 5 description: 'No change made'.
	self assert: zipper safeRight value error = #rightAtRootNode.

	self
		assert: (ZTree value: 5 children: {ZTree value: 2. ZTree value: 3}) printString
		equals: zipper root printString.
! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/20/2011 16:14'!
testChangeToReplacesValue
	| tree zipper |
	tree := self triangleTree.
	zipper := (TreeZipper on: tree) down.
	
	zipper := zipper changeTo: (ZTree value: 5).
	self assert: zipper focus value = 5 description: 'No change made'.
	self assert: zipper right focus value = 3 description: 'Position changed'.

	self
		assert: (ZTree value: 1 children: {ZTree value: 5. ZTree value: 3}) printString
		equals: zipper root printString.! !

!TreeZipperTest methodsFor: 'testing - copying' stamp: 'fbs 2/25/2011 08:06'!
testCopyReturnsExactCopy
	| copy zipper |
	zipper := (self makeZipperOn: self triangleTree) down.
	copy := zipper copy.
	
	self deny: copy == zipper.
	self assert: copy focus = zipper focus.
	self assert: copy up focus = zipper up focus.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 15:55'!
testDeleteAtTopFails
	self should: [self emptyTree zipper delete] raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 16:07'!
testDeleteLeafNodeDeletesNode
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}.
	zipper := tree zipper down delete.
	
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 3}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 16:15'!
testDeleteLeafNodeFocusesOnParent
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2}.
	zipper := tree zipper down.
	zipper := zipper delete.
	
	self assert: zipper root = (ZTree value: 1).
	self assert: zipper focus value = 1.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 16:06'!
testDeleteNodeFocusesOnYoungerSibling
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}.
	zipper := tree zipper down delete.
	
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 3}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 17:15'!
testDeleteOldestSiblingFocusesOnYoungerSibling
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3. ZTree value: 4}.
	zipper := tree zipper down.
	self assert: zipper focus value = 2.
	zipper := zipper delete.
	
	self assert: zipper focus value = 3 description: 'Zipper focused on wrong sibling'.
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 3. ZTree value: 4}) description: 'Wrong thing deleted'.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/20/2011 20:32'!
testDeleteOnlyLeafNode
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2}.
	zipper := tree zipper down delete.
	
	self assert: zipper root = (ZTree value: 1).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 17:15'!
testDeleteYoungerSiblingFocusesOnOlderSibling
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3. ZTree value: 4}.
	zipper := tree zipper down right.
	self assert: zipper focus value = 3.
	zipper := zipper delete.
	
	self assert: zipper focus value = 2 description: 'Zipper focused on wrong sibling'.
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 2. ZTree value: 4}) description: 'Wrong thing deleted'.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 1/1/2011 17:16'!
testDeleteYoungestSiblingFocusesOnOlderSibling
	| tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3. ZTree value: 4}.
	zipper := tree zipper down right right.
	self assert: zipper focus value = 4.
	zipper := zipper delete.
	
	self assert: zipper focus value = 3 description: 'Zipper focused on wrong sibling'.
	self assert: zipper root = (ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}) description: 'Wrong thing deleted'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:18'!
testDownOnEmptyTreeSignalsException
	| zipper |
	zipper := TreeZipper on: self emptyTree.
	
	self
		should: [zipper down]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 22:59'!
testDownOnLeafNodeSignalsException
	| zipper |
	zipper := TreeZipper on: self triangleTree.
	zipper := zipper down.

	self
		should: [zipper down]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/30/2010 23:44'!
testDownOnNonEmptyTreeFocusesOnFirstSibling
	| tree zipper |
	tree := self triangleTree.
	zipper := TreeZipper on: tree.
	zipper := zipper down.
	
	self assert: zipper focus = tree children first.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 17:42'!
testDownOrElseCanUseFocus
	| node tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	zipper downOrElse: [:focus | node := focus ].
	
	self assert: node = zipper focus.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 07:50'!
testDownOrElseOnEmptyTreeRunsBlock
	| ran zipper |
	zipper := self emptyTree zipper.
	ran := false.
	zipper := zipper downOrElse: [ran := true].
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:18'!
testDownOrElseOnLeafNodeRunsBlock
	| ran zipper |
	zipper := TreeZipper on: self triangleTree.
	zipper := zipper down.

	ran := false.
	zipper := zipper downOrElse: [ran := true].
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 18:29'!
testDownOrElseOnNonEmptyTreeFocusesOnFirstSibling
	| ran tree zipper |
	tree := self triangleTree.
	zipper := TreeZipper on: tree.
	ran := false.
	zipper := zipper downOrElse: [ran := true].
	
	self assert: zipper focus = tree children first.
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/20/2011 20:31'!
testInsertDownInsertsChild
	| tree zipper |
	tree := ZTree value: 0.
	zipper := tree zipper.
	
	zipper := zipper insertDown: (ZTree value: 1).
	"self should: [zipper up] raise: ZipperNavigation."
	self assert: zipper focus value = 1 description: 'Zipper not focused on new node'.
	self shouldnt: [zipper up] raise: ZipperNavigation.
	
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 1}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/20/2011 16:17'!
testInsertDownOnEmptyTreeIsATreeWithOneNode
	| tree zipper |
	tree := ZTree empty.
	zipper := tree zipper.
	
	zipper := zipper insertDown: (ZTree value: 0).
	self assert: zipper focus value = 0 description: 'Zipper not focused on new node'.
	
	self assert: zipper root = (ZTree value: 0).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/20/2011 20:31'!
testInsertLeftInsertsImmediatelyOlderSibling
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1. ZTree value: 2}.
	zipper := tree zipper.
	zipper := zipper down right insertLeft: (ZTree value: 3).
	self assert: zipper focus value = 3 description: 'Zipper not focused on new node'.
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 1. ZTree value: 3. ZTree value: 2}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/20/2011 12:43'!
testInsertLeftInsertsOlderSibling
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1}.
	zipper := tree zipper.
	zipper := zipper down insertLeft: (ZTree value: 2).
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 2. ZTree value: 1}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/20/2011 20:54'!
testInsertLeftOnRootIndicatesFailure
	| tree zipper |
	tree := ZTree value: 0.
	zipper := tree zipper.
	self should: [zipper insertLeft: (ZTree value: 1)] raise: ZipperNavigation! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/20/2011 20:31'!
testInsertRightInsertsYoungerSibling
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1}.
	zipper := tree zipper.
	zipper := zipper down insertRight: (ZTree value: 2).
	self assert: zipper focus value = 2 description: 'Zipper not focused on new node'.
	self assert: zipper root = (ZTree value: 0 children: {ZTree value: 1. ZTree value: 2}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/20/2011 12:44'!
testInsertRightOnEmptyTreeInsertsYoungerSibling
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree empty}.
	zipper := tree zipper.
	zipper := zipper down insertRight: (ZTree value: 2).
	self assert: zipper root = (ZTree value: 0 children: {ZTree empty. ZTree value: 2}).! !

!TreeZipperTest methodsFor: 'testing - mutating' stamp: 'fbs 12/20/2011 20:50'!
testInsertRightOnRootIndicatesFailure
	| tree zipper |
	tree := ZTree value: 0.
	zipper := tree zipper.
	self should: [zipper insertRight: (ZTree value: 1)] raise: ZipperNavigation! !

!TreeZipperTest methodsFor: 'testing - testing' stamp: 'fbs 2/17/2011 07:38'!
testIsEmptyOnEmptyTreeReturnsTrue
	self assert: self emptyTree zipper isEmpty.! !

!TreeZipperTest methodsFor: 'testing - testing' stamp: 'fbs 2/17/2011 07:42'!
testIsEmptyOnNonEmptyTreeAlwaysReturnsFalse
	| zipper |
	zipper := self triangleTree zipper.
	self deny: zipper isEmpty.
	
	self deny: zipper down isEmpty.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 18:48'!
testLeftAtLeftmostNodeSignalsException
	| zipper |
	zipper := (TreeZipper on: self triangleTree) down.
	
	self
		should: [zipper left]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 17:29'!
testLeftAtRootNodeSignalsException
	| zipper |
	zipper := TreeZipper on: self triangleTree.
	
	self
		should: [zipper left]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 18:22'!
testLeftMovesToLeftSibling
	| tree zipper |
	tree := self nestedTriangleTree.
	zipper := TreeZipper on: tree.
	zipper := zipper down right down up.
	
	self assert: zipper focus value = 3 description: 'We followed the wrong path'.

	zipper := zipper left.
	self assert: zipper focus value = 2 description: 'Left didn''t go left'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:17'!
testLeftOrElseAtLeftmostNodeRunsBlock
	| ran zipper |
	zipper := (TreeZipper on: self triangleTree) down.
	
	zipper := zipper leftOrElse: [ran := true].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:17'!
testLeftOrElseAtRootNodeRunsBlock
	| ran zipper |
	zipper := TreeZipper on: self triangleTree.
	
	zipper := zipper leftOrElse: [ran := true].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 17:41'!
testLeftOrElseCanUseFocus
	| node tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	zipper leftOrElse: [:focus | node := focus ].
	
	self assert: node = zipper focus.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 18:34'!
testLeftOrElseMovesToLeftSibling
	| ran tree zipper |
	tree := self nestedTriangleTree.
	zipper := TreeZipper on: tree.
	zipper := zipper down right down up.
	
	self assert: zipper focus value = 3 description: 'We followed the wrong path'.

	ran := false.
	zipper := zipper leftOrElse: [ran := true].
	self assert: zipper focus value = 2 description: 'Left didn''t go left'.
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:34'!
testLeftRightIdempotent
	| tree zipper |
	tree := self flatTree.
	zipper := tree zipper down.
	4 timesRepeat: [zipper := zipper right].
	self assert: zipper focus value = 6.
	
	"Node 2 has 4 younger siblings, so 4 is as far as we can navigate with #right.."
	1 to: 4 do:
		[ :i |
			| original navigated |
			original := navigated :=zipper.
			i timesRepeat: [ navigated := navigated left ].
			i timesRepeat: [ navigated := navigated right ].
			self assert: original focus = navigated focus ]! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/22/2011 13:59'!
testLeftmostMovesToLeftmostSibling
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1. ZTree value: 2. ZTree value: 3.}.
	zipper := TreeZipper on: tree.
	zipper := zipper down right right leftmost.
	
	self assert: zipper focus value = 1 description: 'We followed the wrong path'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 10/17/2011 17:51'!
testNonMutatingNavigationReturnsIdenticalStructure
	| data |
	data := ZTree value: 1 children: {ZTree value: 2 children: {ZTree value: 3. ZTree value: 4}. ZTree value: 5}.
	self assert: data == data zipper down right root.! !

!TreeZipperTest methodsFor: 'testing - instance creation' stamp: 'fbs 12/30/2010 22:19'!
testOn
	| tree zipper |
	tree := self emptyTree.
	zipper := TreeZipper on: tree.
	
	self assert: zipper focus = tree.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 18:49'!
testRightAtRightmostNodeSignalsException
	| zipper |
	zipper := (TreeZipper on: self triangleTree) down right.
	
	self
		should: [zipper right]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 18:42'!
testRightAtRootNodeSignalsException
	| zipper |
	zipper := TreeZipper on: self triangleTree.
	
	self
		should: [zipper right]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:35'!
testRightLeftIdempotent
	| tree zipper |
	tree := self flatTree.
	zipper := tree zipper down.
	
	"Node 2 has 4 younger siblings, so 4 is as far as we can navigate with #right.."
	1 to: 4 do:
		[ :i |
			| original navigated |
			original := navigated :=zipper.
			i timesRepeat: [ navigated := navigated right ].
			i timesRepeat: [ navigated := navigated left ].
			self assert: original focus = navigated focus ]! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:27'!
testRightMovesToRightSibling
	| tree zipper |
	tree := self triangleTree.
	zipper := tree zipper.
	zipper := zipper down right.
	
	self assert: zipper focus = (tree children at: 2).! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:15'!
testRightOrElseAtRightmostNodeRunsBlock
	| ran zipper |
	zipper := (TreeZipper on: self triangleTree) down right.
	
	ran := false.
	zipper := zipper rightOrElse: [ran := true].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:15'!
testRightOrElseAtRootNodeRunsBlock
	| ran zipper |
	zipper := TreeZipper on: self triangleTree.
	
	zipper := zipper rightOrElse: [ran := true].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 17:41'!
testRightOrElseCanUseFocus
	| node tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	zipper rightOrElse: [:focus | node := focus ].
	
	self assert: node = zipper focus.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 18:41'!
testRightOrElseMovesToRightSibling
	| ran tree zipper |
	tree := self triangleTree.
	zipper := tree zipper.
	zipper := zipper down.
	
	ran := false.
	zipper := zipper rightOrElse: [ran := true].
	
	self assert: zipper focus = (tree children at: 2).
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:48'!
testRightStoresValueCorrectly
	"#right didn't store the value correctly, and would cause the right-most node
	value to be written to all the subsequently-visited nodes."
	| tree zipper |
	tree := self nestedTriangleTree.
	zipper := tree zipper.
	"Down, any (valid) combination of right/left, then up: no-op."
	self assert: zipper focus value = zipper down right up focus value.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/22/2011 14:03'!
testRightmostMovesToRightmostSibling
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1. ZTree value: 2. ZTree value: 3.}.
	zipper := TreeZipper on: tree.
	zipper := zipper down rightmost.
	
	self assert: zipper focus value = 3 description: 'We followed the wrong path'.
	self assert: tree printString equals: zipper root printString! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 23:31'!
testRootAtChildNodeReturnsRoot
	| tree zipper |
	tree := self triangleTree.
	zipper := tree zipper down.
	
	self assert: zipper root = tree.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 12/31/2010 23:29'!
testRootAtRootReturnsRoot
	| tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	
	self assert: zipper root == tree.! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:55'!
testSafeChangeReplacesValue
	| attempt tree zipper |
	tree := self triangleTree.
	zipper := (TreeZipper on: tree) down.
	
	attempt := zipper safeChange: [ :z | ZTree value: (z value + 1)].
	self assert: attempt isRight.
	self assert: attempt value focus value = 3 description: 'No change made'.
	self assert: attempt value right focus value = 3 description: 'Position changed'.

	self
		assert: (ZTree value: 1 children: {ZTree value: 3. ZTree value: 3.}) printString
		equals: attempt value root printString.! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:54'!
testSafeChangeToAtTopReplacesValue
	| attempt tree zipper |
	tree := self triangleTree.
	zipper := TreeZipper on: tree.
	
	attempt := zipper safeChangeTo: (ZTree value: 5 children: zipper focus children).
	self assert: attempt isRight.
	self assert: attempt value focus value = 5 description: 'No change made'.
	self deny: attempt value safeRight isRight.
	self assert: #rightAtRootNode equals: attempt value safeRight value error.

	self
		assert: (ZTree value: 5 children: {ZTree value: 2. ZTree value: 3}) printString
		equals: attempt value root printString.
! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:52'!
testSafeChangeToReplacesValue
	| attempt tree zipper |
	tree := self triangleTree.
	zipper := (TreeZipper on: tree) down.
	
	attempt := zipper safeChangeTo: (ZTree value: 5).
	self assert: attempt isRight.
	self assert: attempt value focus value = 5 description: 'No change made'.
	self assert: attempt value right focus value = 3 description: 'Position changed'.

	self
		assert: (ZTree value: 1 children: {ZTree value: 5. ZTree value: 3}) printString
		equals: attempt value root printString.! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:51'!
testSafeDeleteAtTopFails
	| attempt zipper |
	zipper := self emptyTree zipper.
	attempt := zipper safeDelete.
	self deny: attempt isRight.
	self assert: #deleteAtRoot equals: attempt value error.
	self assert: zipper equals: attempt value location.! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:50'!
testSafeDeleteLeafNodeDeletesNode
	| attempt tree |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}.
	attempt := tree zipper down safeDelete.
	
	self assert: attempt isRight.
	self assert: attempt value root = (ZTree value: 1 children: {ZTree value: 3}).! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:50'!
testSafeDeleteLeafNodeFocusesOnParent
	| attempt tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2}.
	zipper := tree zipper down.
	attempt := zipper safeDelete.
	
	self assert: attempt isRight.
	self assert: attempt value root = (ZTree value: 1).
	self assert: attempt value focus value = 1.! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:50'!
testSafeDeleteOldestSiblingFocusesOnYoungerSibling
	| attempt tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3. ZTree value: 4}.
	zipper := tree zipper down.
	self assert: zipper focus value = 2.
	attempt := zipper safeDelete.
	
	self assert: attempt isRight.
	self assert: attempt value focus value = 3 description: 'Zipper focused on wrong sibling'.
	self assert: attempt value root = (ZTree value: 1 children: {ZTree value: 3. ZTree value: 4}) description: 'Wrong thing deleted'.! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:50'!
testSafeDeleteOnlyLeafNode
	| attempt tree |
	tree := ZTree value: 1 children: {ZTree value: 2}.
	attempt := tree zipper down safeDelete.
	
	self assert: attempt isRight.
	self assert: attempt value root = (ZTree value: 1).! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:49'!
testSafeDeleteYoungerSiblingFocusesOnOlderSibling
	| attempt tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3. ZTree value: 4}.
	zipper := tree zipper down right.
	self assert: zipper focus value = 3.
	attempt := zipper safeDelete.
	
	self assert: attempt isRight.
	self assert: attempt value focus value = 2 description: 'Zipper focused on wrong sibling'.
	self assert: attempt value root = (ZTree value: 1 children: {ZTree value: 2. ZTree value: 4}) description: 'Wrong thing deleted'.! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:49'!
testSafeDeleteYoungestSiblingFocusesOnOlderSibling
	| attempt tree zipper |
	tree := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3. ZTree value: 4}.
	zipper := tree zipper down right right.
	self assert: zipper focus value = 4.
	attempt := zipper safeDelete.
	
	self assert: attempt isRight.
	self assert: attempt value focus value = 3 description: 'Zipper focused on wrong sibling'.
	self assert: attempt value root = (ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}) description: 'Wrong thing deleted'.! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:39'!
testSafeInsertDownInsertsChild
	| attempt tree zipper |
	tree := ZTree value: 0.
	zipper := tree zipper.
	
	attempt := zipper safeInsertDown: (ZTree value: 1).
	self assert: attempt isRight.
	self assert: attempt value focus value = 1 description: 'Zipper not focused on new node'.
	self shouldnt: [attempt value up] raise: ZipperNavigation.
	
	self assert: attempt value root = (ZTree value: 0 children: {ZTree value: 1}).! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:39'!
testSafeInsertDownOnEmptyTreeIsATreeWithOneNode
	| attempt tree zipper |
	tree := ZTree empty.
	zipper := tree zipper.
	
	attempt := zipper safeInsertDown: (ZTree value: 0).
	self assert: attempt isRight.
	self assert: attempt value focus value = 0 description: 'Zipper not focused on new node'.
	
	self assert: attempt value root = (ZTree value: 0).! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:38'!
testSafeInsertLeftInsertsImmediatelyOlderSibling
	| attempt tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1. ZTree value: 2}.
	zipper := tree zipper.
	attempt := zipper down right safeInsertLeft: (ZTree value: 3).
	self assert: attempt isRight.
	self assert: attempt value focus value = 3 description: 'Zipper not focused on new node'.
	self assert: attempt value root = (ZTree value: 0 children: {ZTree value: 1. ZTree value: 3. ZTree value: 2}).! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:38'!
testSafeInsertLeftInsertsOlderSibling
	| attempt tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1}.
	zipper := tree zipper.
	attempt := zipper down safeInsertLeft: (ZTree value: 2).
	self assert: attempt isRight.
	self assert: attempt value root = (ZTree value: 0 children: {ZTree value: 2. ZTree value: 1}).! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:38'!
testSafeInsertLeftOnRootIndicatesFailure
	| attempt tree zipper |
	tree := ZTree value: 0.
	zipper := tree zipper.
	attempt := zipper safeInsertLeft: (ZTree value: 1).
	self deny: attempt isRight.
	self assert: #insertLeftAtRoot equals: attempt value error.
	self assert: zipper = attempt value location.! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:37'!
testSafeInsertRightInsertsYoungerSibling
	| attempt tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1}.
	zipper := tree zipper.
	attempt := zipper down safeInsertRight: (ZTree value: 2).
	self assert: attempt isRight.
	self assert: attempt value focus value = 2 description: 'Zipper not focused on new node'.
	self assert: attempt value root = (ZTree value: 0 children: {ZTree value: 1. ZTree value: 2}).! !

!TreeZipperTest methodsFor: 'testing - safe mutating' stamp: 'fbs 2/5/2012 21:36'!
testSafeInsertRightOnRootIndicatesFailure
	| attempt tree zipper |
	tree := ZTree value: 0.
	zipper := tree zipper.
	attempt := zipper safeInsertRight: (ZTree value: 2).
	self assert: attempt isLeft.
	self assert: #insertRightAtRoot equals: attempt value error.
	self assert: attempt value location = zipper.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:26'!
testUpAtRootNodeSignalsException
	| zipper |
	zipper := self triangleTree zipper.
	
	self
		should: [zipper up]
		raise: ZipperNavigation.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:14'!
testUpOrElseAtRootNodeRunsBlock
	| ran zipper |
	zipper := self triangleTree zipper.
	
	ran := false.
	zipper := zipper upOrElse: [ran := true.].
	
	self assert: ran.
	self assert: zipper == ran description: 'Unexpected result'.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/16/2011 17:41'!
testUpOrElseCanUseFocus
	| node tree zipper |
	tree := self emptyTree.
	zipper := tree zipper.
	zipper rightOrElse: [:focus | node := focus ].
	
	self assert: node = zipper focus.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:20'!
testUpOrElseReturnsToParentNode
	| ran tree zipper |
	tree := self triangleTree.
	zipper := tree zipper.
	ran := false.
	zipper := zipper down upOrElse: [ran := true].
	
	self assert: zipper focus = tree.
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 2/13/2011 19:21'!
testUpOrElseReturnsToParentNode2
	| ran tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1 children: {ZTree value: 2}}.
	zipper := tree zipper.
	ran := false.
	zipper := zipper down down up upOrElse: [ran := true].
	
	self assert: zipper focus = tree.
	self deny: ran.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 15:26'!
testUpReturnsToParentNode
	| tree zipper |
	tree := self triangleTree.
	zipper := tree zipper.
	zipper := zipper down up.
	
	self assert: zipper focus = tree.! !

!TreeZipperTest methodsFor: 'testing - navigating' stamp: 'fbs 1/1/2011 14:46'!
testUpReturnsToParentNode2
	| tree zipper |
	tree := ZTree value: 0 children: {ZTree value: 1 children: {ZTree value: 2}}.
	zipper := tree zipper.
	zipper := zipper down down up up.
	
	self assert: zipper focus = tree.! !

!TreeZipperTest methodsFor: 'testing - sharing' stamp: 'fbs 11/1/2011 17:24'!
testZipperShouldShareSubstructures
	| t newTree |
	t := self nestedTriangleTree.
	newTree := (t zipper down changeTo: 5) root.
	self assert: newTree children second == t children second.! !

!TreeZipperTest methodsFor: 'fixtures' stamp: 'fbs 12/31/2010 16:53'!
triangleTree
	" Best viewed in a monospaced font:
	   1
	  / \
	 2   3 "
	| b c |
	b := ZTree value: 2.
	c := ZTree value: 3.
	^ ZTree value: 1 children: {b. c.}! !

TestCase subclass: #ZTreeTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Zippers-Tests'!

!ZTreeTests methodsFor: 'testing' stamp: 'fbs 4/24/2011 21:47'!
expectedFailures
	^ #(testAsDotFileOfDoubleNodeTreeShowsGraph.
	testAsDotFileOfEmptyTreeShowsEmptyGraph.
	testAsDotFileOfGrandchildTreeShowsGraph.
	testAsDotFileOfSingleNodeTreeShowsEmptyGraph)! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 4/23/2011 22:41'!
testAsDotFileOfDoubleNodeTreeShowsGraph
	self assert: ((ZTree value: 0 children: {ZTree value: 1}) asDotFile) = ('digraph {', (Character cr), '0->1', (Character cr), '}').! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 4/23/2011 22:39'!
testAsDotFileOfEmptyTreeShowsEmptyGraph
	self assert: (ZTree empty asDotFile) = ('digraph {', Character cr, '}').! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 4/23/2011 22:55'!
testAsDotFileOfGrandchildTreeShowsGraph
	| lines |
	lines := OrderedCollection with: 'digraph {'.
	lines add: '2->3'.
	lines add: '0->1'.
	lines add: '0->2'.
	lines add: '}'.
	
	self assert: ((ZTree value: 0 children: {ZTree value: 1. ZTree value: 2 children: {ZTree value: 3}}) asDotFile) = (lines reduce: [:acc :each | acc , Character cr, each]).! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 4/23/2011 22:39'!
testAsDotFileOfSingleNodeTreeShowsEmptyGraph
	self assert: ((ZTree value: 0) asDotFile) = ('digraph {', Character cr, '}').! !

!ZTreeTests methodsFor: 'testing - converting' stamp: 'fbs 4/28/2011 21:33'!
testAsMutableTreeEmptyTree
	| t |
	t := ZTree empty.
	self assert: t asMutableTree isEmpty.! !

!ZTreeTests methodsFor: 'testing - converting' stamp: 'fbs 4/28/2011 20:59'!
testAsMutableTreeMultipleChildren
	| t |
	t := (ZTree value: 0 children: {ZTree value: 1. ZTree value: 2}) asMutableTree.
	self assert: t notNil.
	self assert: t value = 0.
	self assert: t left notNil.
	self assert: t left value = 1.
	self assert: t left left isEmpty.
	self assert: t left right isEmpty.
	self deny: t right isEmpty.
	self assert: t right value = 2.
	self assert: t right left isEmpty.
	self assert: t right right isEmpty.! !

!ZTreeTests methodsFor: 'testing - converting' stamp: 'fbs 4/28/2011 20:59'!
testAsMutableTreeSingleChildLeft
	| t |
	t := (ZTree value: 0 children: {ZTree value: 1}) asMutableTree.
	self assert: t notNil.
	self assert: t value = 0.
	self deny: t left isEmpty.
	self assert: t left value = 1.
	self assert: t left left isEmpty.
	self assert: t left right isEmpty.
	self assert: t right isEmpty.! !

!ZTreeTests methodsFor: 'testing - converting' stamp: 'fbs 4/28/2011 21:20'!
testAsMutableTreeSingleChildRight
	| t |
	t := (ZTree value: 0 children: {ZTree empty. ZTree value: 1}) asMutableTree.
	self deny: t isEmpty.
	self assert: t value = 0.
	self assert: t left isEmpty.
	self deny: t right isEmpty.
	self assert: t right value = 1.
	self assert: t right left isEmpty.
	self assert: t right right isEmpty.! !

!ZTreeTests methodsFor: 'testing - converting' stamp: 'fbs 4/28/2011 21:20'!
testAsMutableTreeSingleNode
	| t |
	t := (ZTree value: 0) asMutableTree.
	self deny: t isEmpty.
	self assert: t value = 0.
	self assert: t left isEmpty.
	self assert: t right isEmpty.! !

!ZTreeTests methodsFor: 'testing - testing' stamp: 'fbs 4/30/2011 17:27'!
testDepth
	self assert: ZTree empty depth = 0.
	self assert: (ZTree value: 1) depth = 1.
	self assert: (ZTree value: 1 children: {ZTree empty.}) depth = 1.
	self assert: (ZTree value: 1 children: {ZTree value: 2}) depth = 2.
	self assert: (ZTree value: 1 children: {ZTree value: 2. ZTree value: 2 children: {ZTree value: 3}}) depth = 3.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:35'!
testEqualsEqualSingleNodesReturnsTrue
	| a b |
	a := ZTree value: 1.
	b := ZTree value: 1.
	self assert: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:36'!
testEqualsEqualTreesReturnsTrue
	| a b |
	a := ZTree value: 1 children: {ZTree value: 2}.
	b := ZTree value: 1 children: {ZTree value: 2}.
	self assert: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:35'!
testEqualsUnequalSingleNodesReturnsFalse
	| a b |
	a := ZTree value: 1.
	b := ZTree value: 2.
	self deny: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:37'!
testEqualsUnequalTreesInChildrenReturnsFalse
	| a b |
	a := ZTree value: 1 children: {ZTree value: 2}.
	b := ZTree value: 1 children: {ZTree value: 1}.
	self deny: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 12/31/2010 23:39'!
testEqualsUnequalTreesInValueReturnsFalse
	| a b |
	a := ZTree value: 1 children: {ZTree value: 2}.
	b := ZTree value: 2 children: {ZTree value: 2}.
	self deny: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 4/27/2011 17:45'!
testEqualsWithEmptyLeftSubtree
	| a b |
	a := ZTree value: 1 children: {ZTree empty. ZTree value: 2}.
	b := ZTree value: 1 children: {ZTree empty. ZTree value: 2}.
	self assert: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 4/27/2011 17:46'!
testEqualsWithEmptyRightSubtree
	| a b |
	a := ZTree value: 1 children: {ZTree value: 2. ZTree empty}.
	b := ZTree value: 1 children: {ZTree value: 2}.
	self assert: a = b.! !

!ZTreeTests methodsFor: 'testing - comparing' stamp: 'fbs 4/27/2011 17:44'!
testEqualsWithEmptySubtreeVersusNoChildren
	| a b |
	a := ZTree value: 1.
	b := ZTree value: 1 children: {ZTree empty}.
	self assert: a = b.! !

!ZTreeTests methodsFor: 'testing - testing' stamp: 'fbs 4/30/2011 15:31'!
testHasChildren
	self deny: ZTree empty hasChildren.
	self deny: (ZTree value: 0) hasChildren.
	self deny: (ZTree value: 0 children: {ZTree empty}) hasChildren.
	self assert: (ZTree value: 0 children: {ZTree value: 1}) hasChildren.! !

!ZTreeTests methodsFor: 'testing - testing' stamp: 'fbs 4/30/2011 17:21'!
testIsBalanced
	self assert: ZTree empty isBalanced.
	self assert: (ZTree value: 0) isBalanced.
	self assert: (ZTree value: 0 children: {ZTree empty}) isBalanced.
	self assert: (ZTree value: 0 children: {ZTree value: 1}) isBalanced.
	self assert: (ZTree value: 0 children: {ZTree empty. ZTree empty}) isBalanced.
	
	self deny: (ZTree value: 0 children: {ZTree value: 1 children: {ZTree value: 2}}) isBalanced.
	self deny: (ZTree value: 0 children: {ZTree empty. ZTree value: 1 children: {ZTree value: 2}}) isBalanced.! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 2/25/2011 21:33'!
testPrintEmptyTreeShowsSpecialSymbol
	self assert: '<>' = ZTree empty printString.! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 12/31/2010 23:32'!
testPrintSingleNodePrintsNoChildren
	self assert: (ZTree value: 1) printString = '(1)'.! !

!ZTreeTests methodsFor: 'testing - printing' stamp: 'fbs 12/31/2010 23:33'!
testPrintTreeNodePrintsChildren
	| t |
	t := ZTree value: 1 children: {ZTree value: 2. ZTree value: 3}.
	self assert: t printString = '(1 (2), (3))'.! !

!ZTreeTests methodsFor: 'testing - instance creation' stamp: 'fbs 2/25/2011 17:55'!
testValueWithEmptyTreeReturnsEmptyTree
	self assert: ZTree empty == (ZTree value: ZTree empty).! !
ZipperContext initialize!
